/* Next available MSG number is   1 */
/*
      (C) Copyright 1986,87,88,89 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*          DSXADI.H

7/5/89 1910 hrs

    Definitions specific to ADI Display Drivers, and to the coordinating
    generic AutoCAD driver DSXADI

	It seem the cc is not consistent in how it aligns longs in
	structures - it sometimes forces alignment on 4 byte boundaries,
	but not in both this code and in dsxega4, so I'm rearranging some
	packets to puts longs naturally on 4 byte boundaries:
	1/31	change pkinit to align pefmodes 
	- also reorganize pktext and pkboxsave.
	2/1 another change in pkboxsave, pkcommand, pkboxsize, pkdcurs
	and pklopen
	2/15 add pkstring and PSTRING as replacements for PCONIN
	2/28 modify pkhlite, pkmodeline, pkcoordline
	4/18 add EF_MFLIP flag to pefmodes
        7/5 renumber configuration packets
*/

#define    INTLEVEL  6                /* Current interface level */

/*  Request codes  */

#define    INIT1     1                /* Initialize parameters 1 */

/*  Screen configuration bits (pconfig) */

#define    CF_TEXT   1                /* Draw text scrolling area */
#define    CF_MENU   2                /* Draw screen menu */
#define    CF_STATUS 4                /* Draw status line */
#define    CF_CURS   8                /* Raster cursor required */
#define    CF_DIGTIZE 0x10            /* DIGTIZE & PDIGITZE are supported */

/* THE FOLLOWING BITS ARE ONLY MEANINGFUL FOR INTERFACE LEVEL 5 AND ABOVE */
#define    CF_ZOUT   0x20             /* Initial display fully zoomed out */
#define    CF_SYNC   0x40             /* SYNC/PSYNC is supported */
#define    CF_SHADE  0x2000           /* Autodesk product is AutoShade */
#define    CF_SKETCH 0x4000           /* Autodesk product is AutoSketch */
#define    CF_ACAD   0x8000           /* Autodesk product is AutoCAD */

/*  Repaint control bits  */

#define    RP_STAT   1                /* Repaint status line */
#define    RP_MENU   2                /* Repaint status line */
#define    RP_SCROLL 4                /* Repaint scroll area */
#define    RP_GRAPH  8                /* Redraw picture */
#define    RP_TEXT   16               /* redraw from hidden text screen */
#define    RP_BS     32               /* ask BS driver to redraw graphics */

/*  Driver mode bits  */

#define    DM_CCECH  1                /* Suppress echo of control C */
#define    DM_HLEN   2                /* Driver can do highlighting */
#define    DM_GROK   4                /* Driver is single screen */
#define    DM_GROKE  8                /* GROK and GROKE are actually a 2-bit
                                          field which maps into graphok.
                                          2 means Apollo model
                                          3 means Sun model
                                      */

/*  Extended function enable bits */

#define    EF_SYNC     1              /* Enable SYNC function */
#define    EF_REDRAW   2              /* Enable REDRAW function */
#define    EF_CMD      4              /* Enable display driver commands */
#define    EF_XPCMD    8              /* Enable driver transparent cmds */
#define    EF_NEWI    16              /* Enable new user interface */
#define    EF_SAVEB   32              /* Request save buffer for display */
#define    EF_FDRAW   64              /* Fast draw routine supplied */
#define    EF_CURS   128              /* Raster cursor functions supported */
#define    EF_SHELL  256              /* Enable driver shell notifications */
#define    EF_DIGTIZ 512              /* Digitizer coord modification ok */
#define    EF_MV    1024              /* at least limited MV support */
#define    EF_DRAG  2048              /* supports drag lists */
#define    EF_BS    4096              /* big screen device */
#define    EF_MFLIP 8192              /* multiscreen flips desired */


/*  Drawmode control bits  */

#define    DR_HILITE 1                /* Draw using highlighting */
#define    DR_NORDRW 2                /* This line NOT a drawing entity */
/* Do not redisplay on REDRAW calls */
/* The following bit was only meaningful for interface level 4.  For PVEC
   calls, DR_PHYSCY indicated "unbiased" co-ordinates which have the origin
   at the bottom left pixel of the physical screen.  The need for this
   co-ordinate system was eliminated with interface level 5. */
#define    DR_PHYSCY 4                /* Use physical (unbiased) Y coords */

/* SYNC and PSYNC special request control bits */

#define SY_APERTURE     1             /* Define a new APERTURE size */
#define SY_HILITE       2             /* Redraw hilighting vectors */
#define SY_PROMPT       4             /* Reissue any outstanding prompt */

/* PVIEWPORT special request control flags */
#define NO_VEC 1                      /* don't send vectors */
#define NO_BS 2                       /* don't handle this as a BS vport */

/* PREVEC rvflags request bits */
#define YES_VEC    1                  /* turn back on vectors */
#define UN_VEC     2                  /* turn off vectors */
#define AGAIN      4                  /* call me again */


/*  Request codes for packet-mode functions  */

#define PINIT    1                    /* Initialize */
#define PTERM    2                    /* Terminate */
#define PCLEAR   3                    /* Clear screen */
#define PDOT     4                    /* Draw dot */
#define PVEC     5                    /* Draw vector */
#define PFILL    6                    /* Fill polygon */
#define PMARK    7                    /* Draw cursor */
#define PCMARK   8                    /* Remove cursor */
#define PCHAR    9                    /* Draw character */
#define PECHAR  10                    /* End character string */
#define PHLITE  11                    /* Highlight menu item */
#define PDHLITE 12                    /* Dehighlight menu item */
#define PMNUCURS 13                   /* Set text cursor to menu area */
#define PTPROMPT 14                   /* Set text cursor to prompt line */
#define PQPLOT  15                    /* Screen dump to printer */
#define PRPEN   16                    /* Read pen */
#define PGOTEXT 17                    /* Go to text screen */
#define PGOTEXTU 18                   /* Go to text screen at user request */
#define PGOGRAPH 19                   /* Go to graphics screen */
#define PWRSPLIT 20                   /* Write to split screen */
#define PMODELINE 21                  /* Write mode line */
#define PCOORDLINE 22                 /* Write coordinate line */
#define PSYNC   23                    /* Synchronise */
#define PREDRAW 24                    /* Local redraw */
#define PCOMMAND 25                   /* Display driver command */
#define PXPCOMD 26                    /* Transparent command */
#define PBOXCLR 27                    /* Clear box */
#define PBOXSIZE 28                   /* Calculate buffer size to save box */
#define PBOXPUSH 29                   /* Push box on save stack */
#define PBOXPOP  30                   /* Pop box from save stack */
#define PTEXT    31                   /* Write raster text onto screen */
#define PLINE    32                   /* Draw line in natural coordinates */
#define PDCURS   33                   /* Draw raster cursor */
#define PCCURS   34                   /* Remove raster cursor */
#define PBSHELL  35                   /* SHELL notification imminent */
#define PASHELL  36                   /* SHELL notification to driver done */
#define PDIGTIZE 37                   /* Filter digitizer coords/picks */
#define POPENVP 38                    /* open a viewport for BS device */
#define PCLOSEVP 39                   /* close a viewport for BS device */
#define PVIEWPORT 40                  /* change BS view into logical space */
#define PCONIN 41                     /* fake keyboard input */
#define PTABCFG 42                    /* returns screen pointing area */
#define PDRAG 43                      /* smart drag function */
#define PLOPEN 44                     /* real coords of BS view */
#define PCLVP 45                      /* MV non-BS viewport clear */
#define PREVEC 46                     /* change NO_VEC status or redraw vp */
#define PSTRING 47		      /* better fake keyboard input */

#define PCFGREC 59                    /* execution time config record */
#define PNEWCFG 60                    /* new configuration record */
#define PCHGCFG 61                    /* change configuration record */
#define PSHOWCFG 62                   /* show configuration record */

/*  Packets for packet-mode requests.  */

/*  Packet for PINIT function  */

struct pkinit {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status */
    short pconfig;                    /* Screen configuration bits */
    short pintlev;                    /* Interface level */
    short pixdots, piydots, pixdotsm; /* Screen component sizes */
    short pmodelinl;                  /* Mode line length */
    short pmnuchars;                  /* Menu box size in characters */
    short pmaxboxes;                  /* Maximum menu size */
    short pmodes;                     /* Display modes */
    short psplit;                     /* Lines in scrolling ares */
    short phwfill;                    /* Hardware fill mode */
    short pixwid, pixhgt;             /* Pixel width and height (ratio) */
    char phidlines, phidcols;         /* Hidden screen lines and columns */
    long pefmodes;                    /* Extended function mode bits */
    short pymenumax;                  /* Screen menu height in pixels */

    /* The following cells need be initialized only if the bit
       EF_NEWI is set in pefmodes. */

    short pnuxdots, pnuydots;         /* Raw screen size in dots */
    short pnuxchars, pnuychars;       /* Raw screen size in characters */
    short pnuxbpc, pnuybpc;           /* Bits per character */

    short ybias;                      /* ULS - LLG y coord */
    long (*pfdraw)();                 /* fast draw */
};

/*  Packet for function-only requests:  PTERM, PECHAR, PTPROMPT, PGOTEXT,
                                        PGOTEXTU, PCCURSL  */

struct pkfonly {
    short pfunc;                      /* Function code */
};


/* Packet for PCLEAR function */

struct pkclear {
    short pfunc;                      /* Function code */
    short prepaint;                   /* Repaint control bits */
    short vpnumber;                   /* which viewport? */
};


/*  Packet for PDOT function  */

struct pkdot {
    short pfunc;                      /* Function code */
    short px, py;                     /* Coordinates to complement */
};


/*  Packet for PVEC and PLINE functions  */

struct pkvec {
    short pfunc;                      /* Function code */
    short pfx, pfy;                   /* From point */
    short ptx, pty;                   /* To point */
    short pcolour;                    /* Colour */
    short pdrawm;                     /* Drawing modes */
    short vpnumber;
};

/*  Packet for the PFILL function  */

struct pkfill {
    short pfunc;                      /* Function code */
    short pcolour;                    /* Colour */
    short pdrawm;                     /* Drawing modes */
    short pnvert;                     /* Number of vertices */
    short pvert[12][2];               /* Vertex table */
};

/*  Packet for the extended PFILL function  */

struct pkfille {
    short pfunc;                      /* Function code */
    short pcolour;                    /* Colour */
    short pdrawm;                     /* Drawing modes */
    short pnvert;                     /* Number of vertices */
    short pvert[12][2];               /* Vertex table */
    short vpnumber;
};

/*  Packet for PMARK and PCMARK functions  */

struct pkmark {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: draw with XCURSOR ? */
    short px, py;                     /* Location */
    short pcursel;                    /* Cursor type */
    short vpnumber;
};

/*  Packet for PCHAR function  */

struct pkchar {
    short pfunc;                      /* Function code */
    short pchar;                      /* Character to output */
};

/*  Packet for PHLITE and PDHLITE functions  */

struct pkhlite {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: send characters ? */
    char *pchars;		      /* pointer to string */
    short pnchars;                    /* Number of characters to highlight */
};

/* Packet for PMNUCURS function  */

struct pkmnucur {
    short pfunc;                      /* Function code */
    short pboxno;                     /* Box number */
    short pnboxes;                    /* Number of boxes */
};

/*  Packet for PQPLOT function  */

struct pkqplot {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: can device do it ? */
};

/*  Packet for PRPEN and PDIGTIZE functions  */

struct pkrpen {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: pen result */
    short px, py;                     /* Pen coordinates */
};

/*  Packet for PGOGRAPH function  */

struct pkgograph {
    short pfunc;                      /* Function code */
    short prepaint;                   /* Repaint status */
};

/*  Packet for PWRSPLIT function  */

struct pkwrsplit {
    short pfunc;                      /* Function code */
    short pchar;                      /* Character to write */
};

/*  Packet for PMODELINE function  */

struct pkmodeline {
    short pfunc;                      /* Function code */
    short pnchars;                    /* Number of characters in line */
    char *ptext;		      /* pointer to string */
    short pcolour;                    /* Current colour */
};

/*  Packet for PCOORDLINE function  */

struct pkcoordline {
    short pfunc;                      /* Function code */
    short pnchars;                    /* Number of characters in line */
    char *ptext;		      /* pointer to string */
};

/*  Packet for PKSYNC function */

struct pksync {
    short pfunc;                      /* Function code */
    short prequest;                   /* Requested operations */
    short paperture;                  /* New aperture or 0 */
    short vpnumber;
};

/*  Packet for PREDRAW function  */

struct pkredraw {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: able to redraw ? */
    short pflush;                     /* Function: 0 = redraw, 1 = flush */
    short vpnumber;
};

/*  Packet for PCOMMAND function  */

struct pkcommand {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: was command recognised ? */
    short pstrlen;                    /* Command length */
    short dummy;		      /* to align char * */
    char *address;                    /* Command string address */
};

/*  Packet for PXPCOMD function  */

struct pkxpcomd {
    short pfunc;                      /* Function code */
    short pstat;                      /* Status: was command recognised ? */
    short pchar;                      /* Command character */
};

/*  Packet for PBOXCLR function  */

struct pkboxclr {
    short pfunc;                      /* Function code */
    short pxl, pyl, pxh, pyh;         /* Box corners */
    short pcolour;                    /* Box colour */
};

/*  Packet for PBOXSIZE function  */

struct pkboxsize {
    short pfunc;                      /* Function code */
    short pxl, pyl, pxh, pyh;         /* Box corners */
    short dummy;                      /* to align long */
    long pbsize;                      /* Buffer size required */
};

/*  Packet for PBOXPUSH and PBOXPOP functions  */

struct pkboxsave {
    short pfunc;                      /* Function code */
    short pxl, pyl, pxh, pyh;         /* Box corners */
    short pheight;                    /* Height actually saved */
    char *pbufaddr;                   /* Save buffer  */
    long pbufl;                       /* Size of buffer in bytes */
    long plocate;                     /* Value passed back in PBOXPOP */
};

/*  Packet for PTEXT function  */

struct pktext {
    short pfunc;                      /* Function code */
    short px, py;                     /* Text location */
    short pattr;                      /* Text attribute bits */
    char *paddress;		      /* text address */
    short ptlen;                      /* Text length */
    short pbgcolor;                   /* Text background colour */
    short pfgcolor;                   /* Text foreground colour */
};

/*  Raster cursor definition  */

struct rcursor {
    short curx, cury;                 /* Overlap point with screen */
    unsigned short curand[16];        /* Mask to AND with bitmap */
    unsigned short curxor[16];        /* Mask to XOR with bitmap */
};

/*  Packet for PDCURS function  */

struct pkdcurs {
    short pfunc;                      /* Function code */
    short px, py;                     /* Cursor draw location */
    short dummy;
    char *pcaddr;
};


/* packet for PASHELL and PBSHELL */
struct pkshell {
    short pfunc;
    short status;                     /* background/foreground status flag */
    short repaint;                    /* flag bits for repaint control */
};


/* packet for POPENVP function */

struct pkovp {
    short pfunc;
    short vpnumber;                   /* the id # for the new viewport */
    short vpxmin;                     /* the corners of the vport in LLG */
    short vpymin;                     /* (ixdots, iydots) pixel coords */
    short vpxmax;
    short vpymax;
    short gxdots;                     /* you return the upper right corner */
    short gydots;                     /* of your logical space here */
};


/* Packet for PCLOSEVP function */

struct pkcvp {
    short pfunc;
    short vpnumber;                   /* the id of the viewport to close */
};


/* Packet for PVIEWPORT function */

struct pkview {
    short pfunc;
    short vpnumber;                   /* which viewport to move and where */
    short gxl;                        /* in logical space to move it to */
    short gyl;                        /* specified as the lower left */
    short gxh;                        /* and upper right corner */
    short gyh;
    short master;                     /* master viewport if NO_VEC */
    char vflags;
};


/* Packet for PCONIN function */

struct pkconin {
    short pfunc;
    short conin;                      /* passed to you as 0  */
    short vpnumber;		      /* current vp number FYI */
};


/* Packet for PTABCFG function */

struct pktconfig {
    short pfunc;
    unsigned short llx;
    unsigned short lly;               /* lower left corner */
    unsigned short urx;
    unsigned short ury;               /* upper right corner */
};


/* Packet for PDRAG function */

struct  pkdrag {
    short pfunc;
    short pfx,pfy;                    /* from */
    short ptx,pty;                    /* to */
    short ptcolor;                    /* color */
    short dmode;
    short vpnumber;
};


/* Packet for PLOPEN function */

struct  pklopen {       
    short pfunc;
    short vpnumber;                   /* which viewport? */
    short projid;                     /* projection id # */
    short dummy;
    double rxl,ryl;                   /* lower left corner */
    double rxh,ryh;                   /* upper right corner */
};


/* Packet for PCLVP function */

struct pkclvp   {
    short pfunc;
    short vpxmin;                     /* coords in LLG pixels */
    short vpymin;                     /* of rectangle to erase */
    short vpxmax;
    short vpymax;
};


/* packet for prevec function */
struct pkrvec {
    short pfunc;
    short rvflags;
    short vpnumber;
    short master;
};

/* packet for PSTRING */
struct pkstring {
    short pfunc;
    short vpnumber;		      /* current vp FYI */
    char *constr;		      /* init'd to NULL */
};

struct pkcfgrec {                     /* executor portion of adi driver */
    short pfunc;                      /* function code */
    short preclen;                    /* configuration record length */
    char *pcfgrec;                    /* configuration record pointer */
    char paliasid;                    /* alias id */
};

struct pkconfig {                     /* config portion of adi driver */
    short pfunc;                      /* function code */
    short preclen;                    /* configuration record length */
    char *pcfgrec;                    /* configuration record pointer */
};


