/*	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/
/* 8/2/89 1800  hrs

                            dsxega4.c

This sample driver can be manually configured for a 256k VGA with VGA
display or for a 256K EGA with EGA display. It is, of course, possible
to automatically do this configuration, but the manual process is used
here as a test and an example of the use of a driver-local configuration
record stored by AutoCAD. Also note that although this is a display list
driver, it doesn't do smart memory management. A production driver will
need to be more intelligent.

Although this driver contains code to support the EF_MFLIP feature,
AutoCAD ignores this feature now. The feature may (or may not) be
revived in a future version of AutoCAD. I'll post another version of
this driver without the EF_MFLIP code on Compuserve in the near future.

2/18 improve error handling
2/19 background drawing
2/24 fix scrolling text, text font
2/25 add getpriv()
2/28 update PHLITE PDHLITE PMODELINE and PCOORDLINE
fix check mark and fix viewport clear
3/7 hook up execution time cfg for test purposes
3/21 start hooking up ega/vga configuration
3/22 more config
3/31 debugging ega/vga
4/1 start fixing HIGH C errors -
had to kill function prototypes
4/19 add EF_MFLIP flag
- rename entry point to acalls for SCO loader
4/21 start expanding display list limits
4/22 start hooking up AutoCAD-provided config functions
4/27 ifdef in tests of all config functions
4/28 fix clear screen
5/1 debug colors
5/3 start merging polygon fill code
5/4 more polygon fill
5/6 debugging ploygon hilighting
5/7 debugging menu highlighting
5/15 debugging configuration details
5/25 debugging initial crash
5/26 start on bugs adirts51-59
5/31 PVEC needs to be able to draw highlighted XOR vectors
6/1 fix packet tracing for MODELINE and COORDLINE
6/5 don't attempt redraw if list is empty 
6/6 debugging polygon fill failure for thin polylines
6/10 debugging termination problem
6/11 turn off debugging code
6/12 cleanup a bit
7/27 add EF_MFLIP note, remove some debugging messages
8/2 free segsav on PTERM
*/

/* #define configtest  for testing service functions */

/* #define serial */    /* for debugging from serial terminal */

/* #define debug */
/* #define DEBUG */             /* for debugging lflood() */

#define SMART 1

#define trace 1                 /* to connect packet tracing 
                                which is toggled on/off with ^L */
#define flog 1                   /* enables logging to file whose
                                 name is hardcoded below */
#define lfname "test"             /* log file name */

#include    <stdio.h>
#include    <malloc.h>
#include    <sys/types.h>
#include    <sys/signal.h>
#include    <sys/vtkd.h>
#include    <sys/machdep.h>
#include    "dsxadi.h"
/* added include files for rasta2 */

#include "stdio.h"
#include "math.h"
#include "assert.h"
#include "rasta2.h"

#ifndef min
#define min(a, b)   ((a) < (b) ? (a) : (b))
#endif

#ifndef max
#define max(a, b)   ((a) > (b) ? (a) : (b))
#endif

#define V	    (void)
#define vertex poly.pt
#define HIGHVAL 32767

#ifdef serial
FILE *sterm;
#endif

#define FALSE       0
#define FAIL        0
#define TRUE        1
#define PASS        1

#define LASTMEM 0x8000
#define XSIZE       640
#define NBGRAPH     4096
#define CHARHGT cfr.charhgt
#define NOGL        0L                /* No glyph--a Long pointer to 0:0 */
#define TEXTW       80                /* Width of screen in characters */
#define XRIGHT      639               /* Rightmost dot number */
#define BPR         ((XRIGHT + 1 + 7) / 8) /* Bytes per graphic row */
#define MCHARS      8                 /* Characters in a menu box. */
#define CHARWID     8                 /* Character width. */
#define XMENUMIN    (XSIZE - CHARWID*MCHARS) /* Left margin of menu box. */
#define XMAX        (XMENUMIN - 5)    /* Right margin of graphics area. */
#define SPLIT       3                 /* Lines in split-text area. */
#define SPLITHGT    (cfr.charhgt*SPLIT + 2) /* Split-text area height. */
#define YSPLIT      (cfr.ysize - SPLIT*cfr.charhgt) /* Row nr of text area */
#define XCLINE      (CHARWID * 40)    /* Start of coordinate display line. */
#define DEFATTR     7                 /* Attribute of video character */

#define TWOPLANES   (2 * (XSIZE * cfr.ysize) / 8) + 96
                                      /* Bytes req'd to save 2 vGA bit planes
                                      plus extra bytes for raster cursor
                                      (one extra scan line worth) */ 

#define P ((struct PT *)pkt)

#define    BLACK    0
#define    RED      1
#define    YELLOW   2
#define    GREEN    3
#define    CYAN     4
#define    BLUE     5
#define    MAGENTA  6
#define    WHITE    7

/* Logical colors used for dialogue boxes, pull-down menus, etc.
   These color values will only be used for routines dstext(),
   dsline() and dsbox().

   When assigning actual colors to these logical color values, the
   rules are as follows:

1.  BGLCOLOR, TBGLCOLOR, TXTLCOLOR and BRDLCOLOR are simply the colors
    chosen for the corresponding parts of the screen.

2.  For the ALERT, DIALOGUE BOX, MENU-BAR and POP-UP-MENU color triples,
    the background color must differ from the other two.  The border
    color must differ from BGLCOLOR.

3.  LINELCOLOR must differ from DBGLCOLOR and ABGLCOLOR.  If
    convenient, it should also differ from DFGLCOLOR and AFGLCOLOR.

4.  It usually looks gaudy if you pick too many different colors.  Don't
    hesitate to use the same color for multiple purposes.

5.  If you only have two colors (e.g., black and white) then the
    entire color scheme is determined by your choice of BGLCOLOR.  In
    this case, the only scheme which conforms to the rules is:

            a. All background colors are the same as BGLCOLOR.
            b. All other colors differ from BGLCOLOR.

    The #defines for ISBGLCOLOR become particularly convenient.

6.  Currently, only the "background" colors BGLCOLOR, ABGLCOLOR,
    PBGLCOLOR, MBGLCOLOR and DBGLCOLOR get passed to routine dsbox().
    If they are all the same, then the driver code can ignore the
    passed color value for routine dsbox().
*/
#define    BGLCOLOR     (-2)          /* Graphics area background color */
#define    TBGLCOLOR    (-3)          /* Background color for text areas */
#define    TXTLCOLOR    (-4)          /* Color of text */
#define    BRDLCOLOR    (-5)          /* Color for border lines */

#define         BGLCLR  (0)
#define         FGLCLR  (-1)
#define         BRDLCLR (-2)

#define    ALERTLCLR    (-6)
#define    ABGLCOLOR    (ALERTLCLR + BGLCLR) /* Alert box background */
#define    AFGLCOLOR    (ALERTLCLR + FGLCLR) /* Alert box foreground */
#define    ABRDLCOLOR   (ALERTLCLR + BRDLCLR) /* Alert box border */

#define    MBARLCLR     (ALERTLCLR - 3)
#define    MBGLCOLOR    (MBARLCLR + BGLCLR) /* Menu bar background */
#define    MFGLCOLOR    (MBARLCLR + FGLCLR) /* Menu bar foreground */
#define    MBRDLCOLOR   (MBARLCLR + BRDLCLR) /* Menu bar border */

#define    POPLCLR      (MBARLCLR - 3)
#define    PBGLCOLOR    (POPLCLR + BGLCLR) /* Pop-up menu background */
#define    PFGLCOLOR    (POPLCLR + FGLCLR) /* Pop-up menu foreground */
#define    PBRDLCOLOR   (POPLCLR + BRDLCLR) /* Pop-up menu border */

#define    DIALLCLR     (POPLCLR - 3)
#define    DBGLCOLOR    (DIALLCLR + BGLCLR) /* Dialogue box background */
#define    DFGLCOLOR    (DIALLCLR + FGLCLR) /* Dialogue box foreground */
#define    DBRDLCOLOR   (DIALLCLR + BRDLCLR) /* Dialogue box border */
#define    LINELCOLOR   (DIALLCLR - 3)/* Dialogue box line drawing color */

#define    MINLCOLOR    LINELCOLOR    /* Most negative color number */

/* The following #define is useful for (monochrome) display drivers which
   use the same background color for everything.  It generates an
   array which can easily be tested for background colorness */
#define DECLBGARRAY isbglclr[1-MINLCOLOR]={1,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0}
#define ISBGLCOLOR(colour)      ((colour) <= BLACK && isbglclr[-(colour)])

/*  Raster text attributes (for dialogue boxes and pop-up menus)  */
 
#define TANORM   0                    /* Normal display */
#define TAINV    1                    /* Reverse video */
#define TAGREY   2                    /* Greyed out */
#define NOGREY  0                     /* "h" value for normal display */
#define GREY1   1                     /* "h" value for dim or grey display */
#define GREY2   2                     /* "h" value for top visibility */

/******  Global symbols exported  ******/
int intpar();
extern char bgc;                      /* background color */
int (*fdpoint)();                     /* fastdraw pointer */
short overflow = 0;                   /* = 1 if some list overflowed */
short graphx, graphy;                 /* Graphics area width and height. */
short cline;                          /* Status line configured */
short hercws = TRUE;                  /* White screen desired ? */
short yb;                             /* To convert shorts to hard coords */
short graphy;                         /* Graphics area height (pixels). */
short color;                          /* Current colour */
short colormon=0;                     /* set to 1 for color display */
long aybias;                          /* To convert coordinates to screen */
short ybias;
short nuydots;                        /* screen height in pixels */
char color0;                          /* graphic background color */
char colormap[8];                     /* basic acad colors */
long drawflag = 0;                    /* RAM semaphore to lock screen */
char egamem;                          /* EGA regen buffer size, 0 0 -> 64K
                                                                0 1 -> 128K
                                                                1 0 -> 192K
                                                                1 1 -> 256K */
char mdrg[3] = {0, 1, 2};             /* MODE register for write modes 0-2 */
short ovrcolor;                       /* Overscan (outside edges) color */
extern long curam[16], curxm[16];     /* raster cursor and, xor patterns 
                                      declared long to allow room for
                                      shifting bits to allow for partial
                                      characters overlapping the cursor */
unsigned short wtranv[16];            /* for debugging */
short targetv;                        /* for debugging */
char chreg;


/******  Local stuff  ******/

struct svector {
    struct spoint f;
    struct spoint t;
};

struct edg {
    scrcoord ymax, ymin, xpos, xend;
    double slope;
};

static struct svector svec;
static void (*vectaker)();            /* where to send vectors */
static void pixvec();                 /* regular pixel non-dwg vectortaker */
static void pixxvec();                /* xor vector taker */
static void pixlvec();                /* logical vectortaker */
static void drawpoly();
static void lflood();
static short fillcolor;               /* polygon color */
static short fprop;                   /* polygon "property" value */
static short fillhigh;                /* counts every 5th vector for hilight */
static struct spolygon poly;

/* for debugging polys */
#ifdef debug
void dumppoly();
static void dumpsvec();
#endif

/*===============================================*/
static unsigned char chartab[4096];
static unsigned char old_mod;
#define BAD 1
short hilight;
void transfer();
void emancipate();
static void backflip();
extern void fdraw();
/* static void fdraw(); */
/*=============================================*/
#define DLSIZE (100 * 1024) + 2       /* size of each viewport's display
                                      list segment */
#define DDLSIZE (63 * 1024)           /* drag display list size */
#define TEXTSCR 0
#define GRAPHSCR 1
short prevscr = TEXTSCR;
short foreground = 1;
long ecurwind;
/*-------------------------------- end new locals -----------*/

void sendhex();
void senddec();
void sendlong();

#ifdef flog
void dumppkt();
void dumpregs();
void logchar();
void closelog();
void sendlog();
#endif

#ifdef trace
static short tracing = FALSE;        /* controls packet tracing */
#endif

#ifdef flog
FILE *logfile;
#endif

/* Dialogue boxes control */
char *slctr[4];                       /* for saving boxes */
static long nextfree;         /* next free byte in text page */
static short pushed;                  /* # levels of boxes pushed */

static short tcursx = 0;              /* Text cursor position */
static short tcursy = 0;              /* Text cursor position */
static short splitchar = 0;             /* Characters in current line. */
static short splitline;               /* Line we're writing on. */
static short splitdirty = FALSE;        /* Any characters on line flag */
#define splitclean !splitdirty
static short notxt, nomnu;            /* Configuration status */
static char topy;                     /* Highest drawing area Y coordinate */
static short ixdots;                  /* Horizonal graphics size minus one */
static short iydots;                  /* Vertical graphics size minus one */
static short ixdotsm;                 /* Horizontal pixels minus one */
static short modelinl;                /* Mode line length */
static short mnuchars;                /* Character width of menu box */
static short maxboxes;                /* Boxes in menu bar */
static short hwfill;                  /* Hardware fill */
static char textstr[82];              /* Local text string buffer */
static short giflag;                  /* Graphics init flag */
static char *segsav[2];               /* for screen saves */
char *vidbuf;
static char *sglyph;
static char *dglyph;
#define property(c, h, x) (((c) << 3) | ((h) << 1) | (x))
static unsigned props[16 - MINLCOLOR];/* Color to property mapping array */
#define clrprop (props - MINLCOLOR)
#define proptbg (clrprop[TBGLCOLOR])  /* Text background property value */
#define propbg  (clrprop[BGLCOLOR])   /* Graphics area background property */
#define proptxt (clrprop[TXTLCOLOR])  /* Text area foreground property */
#define propbrd (clrprop[BRDLCOLOR])  /* Border lines property value */
static unsigned short prop;                 /* Assembly code drawing property */
char tc[2];                           /* used in cleartext() */

char ltop [256 - MINLCOLOR] = {/* logical to physical colour map */
             /* -18 -17 -16 -15 -14 -13 -12 -11 -10 -09 -08 -07 -06 -05 -04 -03 -02 -01 */
             5, 15,  0,  2, 13, 13,  7, 13, 13,  7, 15,  1,  7,  9,  7,  5, 15,  0,
             /* 00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15 */
             15, 1,  2,  3,  4,  5,  6,  0,  8,  7,  1,  1,  1,  1,  9,  9,
             9,  9,  8,  8,  1,  1,  1,  1,  9,  9,  9,  9,  8,  8,  1,  1,
             1,  1,  9,  9,  9,  9,  8,  8,  2,  2,  2,  2, 10, 10, 10, 10,
             8,  8,  2,  2,  2,  2,  10, 10, 10, 10, 8,  8,  2,  2,  2,  2,
             10, 10, 10, 10, 8,  8,  3,  3,  3,  3, 11, 11, 11, 11,  8,  8,
             3,  3,  3,  3, 11, 11, 11, 11,  8,  8,  3,  3,  3,  3, 11, 11,
             11, 11, 8,  8,  3,  3,  3,  3, 11, 11, 11, 11,  8,  8,  3,  3,
             3,  3, 11, 11, 11, 11,  8,  8,  4,  4,  4,  4, 12, 12, 12, 12,
             8,  8,  4,  4,  4,  4, 12, 12, 12, 12,  8,  8,  4,  4,  4,  4,
             12, 12, 12, 12, 8,  8,  5,  5,  5,  5, 13, 13, 13, 13,  8,  8,
             5,  5,  5,  5, 13, 13, 13, 13,  8,  8,  5,  5,  5,  5, 13, 13,
             13, 13, 8,  8,  5,  5,  5,  5, 13, 13, 13, 13,  8,  8,  5,  5,
             5,  5, 13, 13, 13, 13,  8,  8,  6,  6,  6,  6, 14, 14, 14, 14,
             8,  8,  6,  6,  6,  6, 14, 14, 14, 14,  8,  8,  6,  6,  6,  6,
             14, 14, 14, 14, 8,  8,  1,  1,  1,  1,  9,  9,  9,  9,  8,  8,
             1,  1,  1,  1,  9,  9,  9,  9,  8,  8,  0,  0,  8,  8,  7,  7 };

static char ctext[90];                /* command text string */
static char *cpoint;                  /* pointer into ctext */

#ifdef SMART
static short cflag = -1;                /* command flag */
static char rbuff[18];                /* buffer for real -> string */
#endif

static double rwidth;                 /* width of viewport */
static double rheight;                /* height of viewport */

/* structure for local pan and zoom commands */

#ifdef SMART
static struct   {
    char     *cmdname;
    short dxl, dyl, dxh, dyh;
}   zptab[] = {{"LEFT", -1,  0,  -1, 0}, {"RIGHT",  1,  0,  1, 0},
     {"UP",    0,  1,   0, 1}, {"DOWN",   0, -1,  0, -1},
     {"IN",    1,  1, -1, -1}, {"OUT",   -2, -2,  2, 2},
    };
#endif

/* this structure holds everything you ever wanted to know about
a viewport */
struct wstat {
    short vpnumber;                   /* we'll use 0 to indicate unused vp */
    short projid;                     /* projection id */
    short vpxmin;                     /* viewport position in screen */
    short vpymin;                     /* pixels */
    short vpxmax;
    short vpymax;
    short wixdots;                    /* size of viewport (for use by clip)*/
    short wiydots;
    short lgxdots;                    /* size of logical space */
    short lgydots;
    short lgxl;                       /* viewport position in */
    short lgyl;                       /* logical space */
    short lgxh;
    short lgyh;
    double vrxl;                      /* viewport position in */
    double vryl;                      /* projection space */
    double vrxh;
    double vryh;
    double rxl, ryl;                  /* lower left corner in reals */
    double rxh, ryh;                  /* upper right corner */
    char *dlist;                      /* selector for display list */
    char *ddlist;                     /* drag display list */
    short master;                     /* if slave, master viewport id */
    short slaves;                     /* if master, number of slaves */
    short vectors;                    /* vector status */
    short vxdots;                     /* size of current viewport */
    short vydots;                     /* in logical space */
    short dstate;                     /* drag state */
    short drx;                        /* drag offset */
    short dry;
    int ioffset;                      /* offset into display list of
                                      place to insert next entry */
    unsigned short drs;               /* how much drag image shown */
    unsigned short dioffset;          /* offset into drag display list */
    short overflow;                   /* = 0 normally, 1 if list overflowed */
    short dummy;
};
struct wstat vpinfo[32] = {0};        /* remember up to 16 real viewports,
and up to 15 slave viewports and one fake vp for polygon fills */

/* notes on use of .master .vectors and .slaves in vpinfo 

.slaves      meaning
---------------------------------------
   0         this is not a master viewport
   1         this is a master with one slave
   2         this is a master with 2 slaves
   3         this is a master with 3 slaves

.master      meaning
-------------------------------------------
  0          this is not a slave viewport
 >0          master vpnumber (this is a slave)

.vectors     meaning
----------------------------------------------
  -1         tell PREVEC to turn off vectors
   0         normal
   1         tell redraw to send vectors
   2         tell PREVEC to turn on vectors
   3         tell redraw to send vectors & PREVEC to turn 'em on
*/



/* DRAG NOTES:
vpinfo[i].drx and vpinfo[i].dry are the current logical offset of
the drag image from its base position.

vpinfo[i].drs is a relative pointer into the viewport's drag list,
indicating how many words of the list have been processed by dredraw
(for large drag shapes, we draw some and then return to acad to see
if there is user input)

vpinfo[i].dstate is a bit-coded set of flags. 
    bit 0 when set indicates a drag list overflow for that viewport
    bit 1 when set indicates a viewport initialized for logical dragging
                   (otherwise viewport-independent physical pixels)
    bit 4 when set indicates we are in the middle of redrawing a large
                   drag image. When we get a dmode=4 call and this bit
                   is set, we know acad didn't have new user input and
                   we should continue redrawing where we left off
*/

/* external functions defined in dsxegaa4.asm */
extern void lvec();
extern void slvec();
extern void rlvec();
extern void luvec();
extern int vtran();
extern int svtran();
/* extern void egasave();
extern void egarest(); */
extern void egacur();
extern void egaccur();
extern void tadsdot();
extern void tadsvec();
extern void tadsglyph();
extern void tbyteblt();
extern void initpal();
extern void fastdraw();
extern void fdsetup();

static unsigned bytesfor();
static unsigned byteof();
static void gograph();
static void wrsplit();
static void wrsplit1();
static void msinit();
static void send();
static void redraw();
static short prevec();
static void plopen();
static void pviewport();
static void closevp();
static void openvp();
static short slave();
static short scale();
static short viewck();
static short stest();
static short dredraw();
static short dappend();
static void dvtext();
static void dvline();
struct pkinit *pkt;



/*  memory map for 256K VGA 

in Mode 0x1b (graphic 640 x 480 x 16 colors)

0xa000:0000 - start of graphic page 
0xa000:9600 - end of 640 x 480 graphics 
              and start of memory used to save raster cursor
0xa000:9630 - end of raster cursor storage

in text Mode 03

0xb800:0000 - start text page 0

*/
#ifdef configtest
static double rnum;
#endif
static int key;                 /* used for intparl */
static int dsadicfg();		/* config entry point */
static int dsadixqt();		/* execution entry point */
struct edevent {
	struct edevname *edevname;		/* link to first name */
	int (*cfgentry)();			/* configuration entry point */
	int (*xqtentry)();			/* execution entry point */
};

struct edevname {
	struct edevname *edevname;		/* link to next name or NULL */
	char dev_aliasid;			/* alias flag */
	char dev_name[42];			/* driver config menu name */
};

static struct edevname ega4name = {NULL, ' ',"EGA ADI display driver"};
static struct edevent dscev = {&ega4name, dsadicfg, dsadixqt};

int (*entvec)(struct genpkt *);	/* service routine entry point */

struct edevent *
/*FCN*/acalls(entry)
  int (*entry)();
{
	entvec = entry;		/* hook up service routines */
	return(&dscev);		/* hook up this driver */
}

struct cfgstr {
	int ysize;
	int charhgt;
	int iopl;
	int gmode;
	int tmode;
        int gfont;
	char type[19];
};
struct cfgstr *cfgptr;
struct cfgstr cfgrec;           /* configuration time configuration record */

struct cfgstr cfvga = {
		480, 16, VGA_IOPRIVL, M_VGA12, M_VGA_80x25, GIO_FONT8x16,
		"VGA \0"};
struct cfgstr cfega = {
		350, 14, EGA_IOPRIVL, M_ENH_CG640, M_ENH_C80x25, GIO_FONT8x14,
		"EGA \0"};
struct cfgstr cfr;              /* execution time configuration record */

#define LASTMEM 0x8000
#define XSIZE       640
static int
/*FCN*/dsadicfg(cfgpkt)
  struct pkconfig *cfgpkt;
{
	switch(cfgpkt->pfunc) {
		case PNEWCFG:
			getcfg();
			cfgpkt->preclen = sizeof(cfgrec);
			cfgpkt->pcfgrec = (char *)cfgptr;
			break;

		case PCHGCFG:
			getcfg();
			cfgpkt->preclen = sizeof(cfgrec);
			cfgpkt->pcfgrec = (char *)cfgptr;
			break;

		case PSHOWCFG:
			memcpy(&cfgrec, cfgpkt->pcfgrec, cfgpkt->preclen);
			mprintf("  %s\n",cfgrec.type);
			break;
	}
	return(1);
}

/* get a configuration choice from the user, offering vga or ega.
ctrl C returns with no change in configuration */
static int
/*FCN*/getcfg()
{
#ifdef configtest
mprintf("config test, print signed # %d\n",-12345);
intparl("intparl test",1, 5, &key);
if (cancel()) {
     mprintf("control C was detected! errbeep follows...\n");
     errbeep();
}
if (eos()) {
    mprintf("null response (eos) detected, mgerr follows...\n");
    mgerr();
}
if (nultrm()) mprintf("nulltrm triggered\n");
key = 0;
mprintf("enter a number from 1 to 10:");
if (!mscani(&key))
	invrsp();          /* test "invalid response */
else
	mprintf("\n you entered %d\n",key);
rnum = 0;
mprintf("test of distpar\n");
distpar(&rnum,"Enter a real number:");
printf("\nyou entered %f\n",rnum);
rnum = 0;
mprintf("test of mscanr - enter a real number:\n");
if (!mscanr(&rnum))
	invrsp();
else
	printf("\nyou entered %f\n",rnum);


#endif
	if (cfgrec.ysize == 350) {
		key = 1;
		cfgptr = &cfega;
	} else {
		key = 0;
		cfgptr = &cfvga;
	}
	mprintf("\n0. %s",cfvga.type);
	mprintf("\n1. %s",cfega.type);
	intpar("\nEnter selection", &key, 1);
	if (cancel()) {        /* user hit ^C */
		if (cfgptr != &cfvga && cfgptr != &cfega)
			cfgptr = &cfvga;   /* default */
	} else {
		switch (key) {
			case 0:
				cfgptr = &cfvga;
				break;
			case 1:
				cfgptr = &cfega;
				break;	
		}
	}
	memcpy(&cfgrec, cfgptr, sizeof(cfgrec));
}
			
static int
/*FCN*/dsadixqt(pkt)
  struct pkinit *pkt;
{
    short wid, p, j, v, fg, dcx, dcy, newpkt;
    int i;
    struct rcursor curp;
    unsigned short fpat;
    long lpat, l;
    short *fetch;
    unsigned short adapter;

#ifdef trace
    if (tracing)
        dumppkt('\0',pkt);
#endif

    newpkt = FALSE;                   /* Packet not modified */
    if (foreground)
        switch (pkt->pfunc) {

	case PCFGREC:
#define PT pkcfgrec
		memcpy(&cfr, P->pcfgrec, sizeof(cfr));
		break;
#undef PT

        case PINIT:                   /* Initialise packet mode */
#define PT pkinit
            P->pstat = FAIL;

#ifdef flog
            logfile = fopen(lfname,"wb");
#endif
            prevscr = TEXTSCR;


#ifdef serial
sterm = fopen("/dev/tty1a","w+");
#endif

			if (!getpriv())	{
				mprintf("unable to gain io privelage\n");
				newpkt = TRUE;
				break;
			}

            adapter = ioctl(1, CONS_CURRENT, (char *)0);
            if (adapter != VGA && adapter != EGA)  {
                perror("No VGA or EGA graphics adapter found or \
Incorrect display.\n");
		newpkt = TRUE;
		break;
	}

	old_mod = ioctl(1, CONS_GET, NULL);

    if (P->pintlev <= 5) {
        mprintf("This driver needs ADI level 6 or greater.\n");
        fflush(stdout);
        newpkt = TRUE;
        break;
    }

    segsav[0] = (char *)malloc(TWOPLANES); /* for saving screens */
    segsav[1] = (char *)malloc(TWOPLANES); /* takes 4 planes total */

    splitline = YSPLIT / cfr.charhgt;
    egamem = 0x03;
    for (i = 16; i--; )
        clrprop[i] = property(i, NOGREY, FALSE); /* Identity mapping */
    notxt = !(P->pconfig & CF_TEXT);
    nomnu = !(P->pconfig & CF_MENU);
    cline = !!(P->pconfig & CF_STATUS);
    topy = cline ? CHARHGT : 0;       /* Highest drawing area Y coordinate */
    ixdotsm = XSIZE - 1;
    ixdots = nomnu ? XSIZE - 1 : XMAX - 1;
    aybias = ybias = iydots = notxt ? cfr.ysize - 1 : cfr.ysize - SPLITHGT - 1;
    if (cline)
        iydots -= CHARHGT;
    modelinl = XCLINE / 8;
    mnuchars = MCHARS;
    maxboxes = nomnu ? 0 : (ybias + 1) / CHARHGT;
    hwfill = 2;                       /* fake hardware fill */

    bgc = property(ltop[0-MINLCOLOR],0 ,0);
    /* remember backgnd color */

    giflag = 1;


    if ((ioctl(1, MODESWITCH | cfr.gmode, (char *)0)) != 0) { /* graphic mode */
	ioctl(1, MODESWITCH | old_mod, NULL);
	perror("graphic mode shift");
	newpkt = TRUE;
	break;
    }
	
    
    vidbuf = (char *)ioctl(1, MAPCONS, (char *)0); /* pointer to screen */

/*
    if ((ioctl(1, cfr.iopl, 1)) != 0)  { 
	perror("ioprivl ioctl");
	newpkt = TRUE;
	break;
	}
*/

	if (ioctl(1, cfr.gfont, chartab) != 0) {
		ioctl(1,MODESWITCH | old_mod, 0);
		mprintf("\nCan't get font from display adaptor!\n");
		newpkt = TRUE;
		break;	
	}

    gograph();
    giflag = 0;

    /*  Draw the borders */
    if (!notxt) {
        tadsvec(0, ybias + 1, XRIGHT, ybias + 1, 0x40);
    }
    if (!nomnu) {
        tadsvec(ixdots + 1, ybias, ixdots + 1, 0, 0x40);
    }

    P->pixdots = ixdots - 1;
    P->piydots = iydots;
    P->pixdotsm = ixdotsm;
    P->pmodelinl = modelinl;
    P->pmnuchars = mnuchars;
    P->pmaxboxes = maxboxes;
    P->pmodes = DM_HLEN | DM_GROK | DM_CCECH;
    P->psplit = SPLIT;
    P->phwfill = hwfill;
    if (cfr.ysize == 350) {
        P->pixwid = 35;
        P->pixhgt = 40;
    } else {    
        P->pixwid = 1;                /* X pixel spacing */
        P->pixhgt = 1;                /* Y pixel spacing */
    }
    P->phidlines = 25;                /* we let autodesk product */
    P->phidcols = 80;                 /* maintain hidden text screen */
    P->pefmodes = EF_NEWI | EF_SHELL | EF_CURS |
    	EF_XPCMD | EF_CMD | EF_MFLIP | 
    	EF_REDRAW | EF_MV | EF_BS | EF_DRAG | EF_FDRAW;
    P->pymenumax = maxboxes * CHARHGT - 1;
    P->pnuxbpc = CHARWID;
    P->pnuybpc = CHARHGT;
    P->pnuxchars = XSIZE/P->pnuxbpc;
    P->pnuychars = cfr.ysize/P->pnuybpc;
    P->pnuxdots = XSIZE;
    P->pnuydots = cfr.ysize;
    P->pstat = PASS;
    P->ybias = ybias;
    P->pfdraw = fdraw;
#undef PT

    /* init all vpinfo numbers to 0 => empty */
    for (i = 0; i < 31; i++) {
        vpinfo[i].vpnumber = 0;
        vpinfo[i].master = 0;         /* init to independent vp */
        vpinfo[i].slaves = 0;         /* with no slaves */
        vpinfo[i].vectors = 0;        /* and normal vector status */
        vpinfo[i].dioffset = 0;       /* empty drag lists */
        vpinfo[i].ddlist = 0;         /* drag lists not allocated */
        vpinfo[i].dlist = 0;          /* display lists not allocated */
    }
    newpkt = TRUE;
    break;

case PTERM:                           /* Terminate */
#ifdef flog
    if (logfile >= (FILE *)0)
        fclose(logfile);
#endif
    ioctl(1, MODESWITCH | old_mod, NULL); /* restore old video mode */
    for (i = 0; i < 16; i++) {
        if (vpinfo[i].vpnumber) {      /* if any viewport was left open */
            free(vpinfo[i].dlist - 2);
            free(vpinfo[i].ddlist);
        }
    }
    for (i = 0; i < 4; i++) {
        if (slctr[i]) {                 /* if box push memory not freed */
            free(slctr[i]);
            slctr[i] = 0;
            pushed = 0;
         }
    }
    for (i = 0; i < 2; i++) {
        if (segsav[i]) {                 /* text flip screen memory */
            free(segsav[i]);
            segsav[i] = 0;
         }
    }
    break;

case PBSHELL:
case PASHELL:
#define PT pkshell
    backflip(P->status);

    switch(P->status)   {

    case 0:                           /* from bg -> fg */
        foreground = 1;
        if (prevscr == GRAPHSCR)
            P->repaint = RP_STAT | RP_MENU | RP_SCROLL | RP_BS;
        else
            P->repaint = RP_TEXT;
        newpkt = TRUE;
        break;

    case 1:                           /* regular shell command */
        foreground = 1;
        break;

    case 2:
    case 3:
        foreground = 0;
        break;

    default:
        break;
    }
#undef PT
    break;

case PCLEAR:                          /* Clear screen */
#define PT pkclear

    if (!P->vpnumber) {
        tadsglyph(0, topy, ixdots, ybias - topy + 1, 0x78, NOGL);
        if (P->prepaint != 0) {
            P->prepaint = 0;          /* Repaint nothing */
            newpkt = TRUE;
        }
    }  else  {
        i = 0;
        while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
        if (vpinfo[i].vpnumber != P->vpnumber)
            break;
        tadsglyph(vpinfo[i].vpxmin, ybias - vpinfo[i].vpymax,
                 vpinfo[i].wixdots, vpinfo[i].wiydots + 1, 0x78, NOGL);
    }
#undef PT
    break;

case PDOT:                            /* Draw dot */
#define PT pkdot
    tadsdot(P->px, P->py);
#undef PT
    break;

case PVEC:                            /* Draw vector */
#define PT pkvec
    if (P->pcolour == -1) {           /* xor vector */
        prop = property(8, (P->pdrawm & DR_HILITE) ? GREY2 : NOGREY, TRUE);
    } else {
        prop = property(ltop[P->pcolour - MINLCOLOR], NOGREY, FALSE);
        if (P->pdrawm & DR_HILITE)
            prop |= property(0, GREY2, FALSE);
    }

    if (!P->vpnumber) {
        tadsvec(P->pfx, ybias - P->pfy, P->ptx, ybias - P->pty, prop);
    } else {
        /* viewport specific, logical vectors */
        if (!(P->pdrawm & DR_NORDRW)) /* if not nondrawing */
            /* it is part of display list */
            lvec(P->pfx, P->pfy, P->ptx, P->pty,
                 prop, P->vpnumber);
        else  {
            /* not part of display list */
            i = 0;
            while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
            if (vpinfo[i].vpnumber != P->vpnumber)
                break;
            if (vpinfo[i].master) {
                svtran(P->vpnumber);  /* if slave */
            } else
                vtran(P->vpnumber);   /* if free */
            /* arrive here with ecurwind initialized */
            slvec(P->pfx, P->pfy, P->ptx, P->pty, prop);
        }
    }    
#undef PT
    break;

case PCMARK:                          /* Remove cursor */
case PMARK:                           /* Draw cursor */
#define PT pkmark
    switch (P->pcursel) {
    case 0:
    case 3:
        prop = property(8, NOGREY, TRUE);
        if (!P->vpnumber) {
            tadsvec(0, ybias - P->py, ixdots, ybias - P->py, prop); /*  -  */
            tadsvec(P->px, topy, P->px, ybias, prop); /*  |  */
            j = PASS;
        } else {
            i = 0;
            while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
            if (vpinfo[i].vpnumber == P->vpnumber) {
                tadsvec(vpinfo[i].vpxmin,ybias - P->py,vpinfo[i].vpxmax,
                       ybias - P->py, prop); /* - */
                tadsvec(P->px,ybias - vpinfo[i].vpymin, P->px,
                       ybias - vpinfo[i].vpymax, prop); /* | */
                j = PASS;
            } else {
                j = FAIL;
            }
        }
        break;
    default:
        j = FAIL;
        break;
    }
    if (P->pstat != j)   {
        P->pstat = j;
        newpkt = TRUE;                /* only rewrite pkt if necessary */
    }
#undef PT
    break;

case PCHAR:                           /* Write character */
#define PT pkchar

    if (tcursx <= (XSIZE-CHARWID)) {
        sglyph = (chartab + (P->pchar * CHARHGT));
        tadsglyph(tcursx, tcursy, CHARWID, CHARHGT, 0x68, NOGL);
        tadsglyph(tcursx, tcursy, CHARWID, CHARHGT, 0x38, sglyph);
    }
    tcursx += CHARWID;
#undef PT
    break;

case PHLITE:                          /* Highlight menu box */
case PDHLITE:                         /* Dehighlight menu box */
#define PT pkhlite
    tadsglyph(tcursx, tcursy, CHARWID * MCHARS, CHARHGT, 0x79, NOGL);
#undef PT
    break;

case PMNUCURS:                        /* Position text cursor to menu box */
#define PT pkmnucur
    tcursx = XMENUMIN;
    tcursy = ((ybias + 1) * P->pboxno + P->pnboxes - 1) / P->pnboxes;
#undef PT
    break;

case PQPLOT:                          /* Screen dump */
#define PT pkqplot

    for (j = 0; j < 16; j++)   {
send("j, vpinfo[j].ioffset;");
senddec(j);
send(",");
senddec(vpinfo[j].ioffset);
send("\n");
        if (vpinfo[j].vpnumber && vpinfo[j].ioffset) {
            send("viewport ");
            senddec(j);
            send("\n");
            for (i = 0; i < vpinfo[j].ioffset; i++) {
                fetch = (short *)vpinfo[j].dlist;
                sendhex(4,*(fetch + i));
                send(" ");
                if (i % 16 == 0)
                    send("\n");
            }
        }
    }

    if (P->pstat != TRUE)   {
        P->pstat = TRUE;              /* */
        newpkt = TRUE;                /* copy packet only if needed */
    }
#undef PT
    break;


case PRPEN:                    
    break;


case PGOTEXT:
case PGOTEXTU:
    sglyph = vidbuf;
    dglyph = segsav[0];
    tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x01, 0); /* plane 0 */
    dglyph += (XSIZE * cfr.ysize) / 8;  /* one bit plane */
    tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x02, 1); /* plane 1 */
    dglyph = segsav[1];
    tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x04, 2); /* plane 2 */
    dglyph += (XSIZE * cfr.ysize) / 8;
    tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x08, 3); /* plane 3 */

    ioctl(1, MODESWITCH | cfr.tmode, (char *)0); /* text mode */
    prevscr = TEXTSCR;
    cleartext();          break;

case PGOGRAPH:                        /* Go to graphics screen */
#define PT pkgograph
#ifndef bug1
    gograph();
    P->prepaint =  RP_SCROLL;
#endif
#undef PT
    newpkt = TRUE;
    break;

case PWRSPLIT:                        /* Write to text scrolling area */
#define PT pkwrsplit
    wrsplit(P->pchar);
#undef PT
    break;

case PMODELINE:                       /* Position to mode line */
#define PT pkmodeline
    tcursx = tcursy = 0;
	if(!(j = P->pnchars))
            break;
	prop = property(ltop[P->pcolour-MINLCOLOR], NOGREY, FALSE);
	tadsglyph(tcursx, tcursy, CHARWID * j, CHARHGT, 0x68, NOGL);
	while ((j-- > 0) && (tcursx <= (XSIZE-CHARWID))) {
		sglyph = (chartab + (*P->ptext++) * CHARHGT);
		tadsglyph(tcursx, tcursy, CHARWID, CHARHGT, 
                     prop == 0x68 ? 0x00 : prop, sglyph);
		tcursx += CHARWID;
	}
		
#undef PT
    break;

case PCOORDLINE:                      /* Position to coordinate line */
#define PT pkcoordline
    tcursx = XCLINE;
    tcursy = 0;
	if(!(j = P->pnchars))
            break;
	tadsglyph(tcursx, tcursy, CHARWID * j, CHARHGT, 0x68, NOGL);
	while ((j-- > 0) && (tcursx <= (XSIZE-CHARWID))) {
		sglyph = (chartab + (*P->ptext++) * CHARHGT);
		tadsglyph(tcursx, tcursy, CHARWID, CHARHGT, 0x38, sglyph);
		tcursx += CHARWID;
	}

#undef PT
    break;

case PREDRAW:
#define PT pkredraw

    if (P->pflush) {
        if (P->vpnumber == 0) {
            for (i = 0; i < 16; i++) {
                vpinfo[i].vectors &= 2;
                vpinfo[i].ioffset = vpinfo[i].overflow = 0;
            }
            break;
        } else {
            i = 0;
            while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
            vpinfo[i].vectors &= 2;   /* we are about to get vectors */
            /* wipe just one viewport */
            vpinfo[i].ioffset = vpinfo[i].overflow = 0;
            break;
        }
    }                                 /* end of P->pflush "if" */

#ifdef serial
vpidump();
#endif

    /* if you get this far, a redraw has been requested */
    if (!P->vpnumber) {
        j = 0;
        for (i = 0; i < 16; i++) 
            if (vpinfo[i].overflow || (vpinfo[i].vectors & 2)) 
                j = 1;
        if (j)   {                    /* someone overflowed */
            for (i = 0; i < 16; i++) {
                vpinfo[i].vectors &= 2; /* throw away entire list */
                vpinfo[i].overflow = vpinfo[i].ioffset = 0;
            }
            P->vpnumber = 0;          /* need complete fresh list */
            P->pstat = 0;             /* couldn't redraw, bad list */
            newpkt = TRUE;
            break;
        }
    }

    i = 0;
    while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;

    if (P->vpnumber)  {               /* asked to redraw a single vp */
        if (vpinfo[i].overflow) {     /* can we do it? */
            vpinfo[i].vectors &= 2;   /* throw away entire list */
            vpinfo[i].overflow = vpinfo[i].ioffset = 0;
            i = -1;
        }
        if (vpinfo[i].vectors & 1)  {
            vpinfo[i].vectors &= 2;
            i = -1;                   /* remember someone needed redraw */
        }
        if (vpinfo[i].master) {
            j = (vtran(vpinfo[i].master) >> 8) - 1;
            if (vpinfo[j].overflow) { /* if our master overflowed */
                i = -1;
                P->vpnumber = 0;      /* force fresh vectors all around */
            }
        }
        if ((i < 0))  {               /* if we need vectors */
            P->pstat = 0;
            newpkt = TRUE;
            break;
        }
    }
    /* if you get this far, you think you can do the redraw */
    /* start redrawing from local display list */
    if (P->vpnumber) {                /* redraw one viewport */
        if (vpinfo[i].master > 0) {   /* if a slave */
            vpinfo[i].vectors = 0;
            j = vtran(vpinfo[i].master); /* for searching dlist */
            svtran(P->vpnumber);      /* but init ecurwind to slave */
            redraw((j >> 8) - 1);
        } else {                      /* not a slave */
            vtran(P->vpnumber);
            redraw(i);
        }
    } else {                          /* redraw everyone in sight! */
        for (i = 0; i < 16; i++) {
            if (vpinfo[i].vpnumber) { /* if active */
                if (vpinfo[i].master > 0) { /* if slave */
                    vpinfo[i].vectors = 0;
                    j = vtran(vpinfo[i].master); /* for searching dlist */
                    svtran(vpinfo[i].vpnumber); /* but init ecurwind to slave */
                    redraw((j >> 8) -1); /* redraw one vp */
                } else {              /* master or regular vp */
                    j = vtran(vpinfo[i].vpnumber);
                    redraw(i);        /* redraw one vp */
                }
            }
        }     
    }
    P->pstat = 1;
    newpkt = TRUE;
#undef PT
    break;


case PCOMMAND:                        /* Display driver command */
#define PT pkcommand
#ifdef  SMART
    P->pstat = FALSE;                 /* Assume an unrecognized command */
    newpkt = TRUE;
    if (P->pstrlen < 1 || P->pstrlen > sizeof(textstr))
        break;
    memcpy(textstr, P->address, P->pstrlen + 1);
    for (i = sizeof(zptab)/sizeof(zptab[0]); i--; ) {
        if (strcmp(textstr, zptab[i].cmdname) == 0) {
            cflag = i;                /* save command */
            P->pstat = TRUE;          /* Found a valid command */
            break;
        }
    }
#endif
    break;
#undef PT


case PXPCOMD:                         /* Display driver transparent command */
#define PT pkxpcomd
#ifdef trace
    if (P->pstat != TRUE)
        P->pstat = newpkt = TRUE;
    /* Use ^L to toggle tracing of packets.  Real drivers can use it
    for whatever they want, or can turn it off entirely. */
    tracing = !tracing;
    send("<ADI packet Tracing ");
    send(tracing ? "on> " : "off> ");
	send("\n");
    vpidump();
#endif
#undef PT
    break;

case PBOXCLR:                         /* Clear box */
#define PT pkboxclr
    if (P->pcolour < 0) {
        prop = property(ltop[P->pcolour - MINLCOLOR], NOGREY, FALSE);
    } else {
        prop = property((P->pcolour % 16), NOGREY, FALSE);
    }
    tadsglyph(P->pxl, P->pyl, P->pxh - P->pxl + 1, P->pyh - P->pyl + 1,
             prop, NOGL);
#undef PT
    break;

case PBOXSIZE:
    break;

case PBOXPUSH:                        /* Save box */
#define PT pkboxsave
    i = bytesfor(P->pxl, (P->pxh - P->pxl + 1)); /* width in bytes */
    l = nextfree;
    j = (LASTMEM - nextfree + 1) / i; /* height of box */
    if (j > P->pyh - P->pyl + 1)
        j = P->pyh - P->pyl + 1;

    wid = BPR - i;                    /* width not to be saved */

    if (!pushed)   {                  /* if no boxes already pushed */
        for (v = 0; v < 4; v++)
            slctr[v] = (char *)malloc(LASTMEM);
    }
    pushed++;
    sglyph = vidbuf + byteof(P->pxl, P->pyl); /* source */
    for (p = 0; p < 4; p++) {
        dglyph = slctr[p] + nextfree; /* destination addr */
        tbyteblt(dglyph, sglyph, i, j, 0, wid, 1 << p, p);
    }
    nextfree += i * j;
    if (j != P->pheight || l != P->plocate) {
        P->pheight = j;
        P->plocate = l;
        newpkt = TRUE;
    }
#undef PT
    break;


case PBOXPOP:                         /* Restore box */
#define PT pkboxsave

    nextfree = P->plocate;
    i = bytesfor(P->pxl, (P->pxh - P->pxl + 1)); /* width */
    j = P->pyh - P->pyl + 1;          /* height */
    wid = BPR - i;                    /* width not included in box */
    pushed--;
    dglyph = vidbuf + byteof(P->pxl, P->pyl); /* source */
    for (p = 0; p < 4; p++) {
        sglyph = slctr[p] + nextfree; /* destination addr */
        tbyteblt(dglyph, sglyph, i, j, wid, 0, 1 << p, p); /* plane p */
    }
    if (!pushed)   {                  /* if no pushed boxes are left */
        for (i = 0; i < 4; i++) {
            free(slctr[i]);
            slctr[i] = 0;
        }
    }
#undef PT
    break;

case PLINE:                           /* Draw line in natural coordinates */
#define PT pkvec
    dvline(P->pfx, P->pfy, P->ptx, P->pty, P->pcolour);
#undef PT
    break;

case PTEXT:                           /* Draw raster text */
#define PT pktext
    dvtext(P->px, P->py, P->pattr, P->ptlen, P->paddress, P->pbgcolor,
           P->pfgcolor);
#undef PT
    break;


    case PFILL:                       /* Fill polygon */
#define PT pkfille
#define VP vpinfo[i]
    poly.npoints = P->pnvert;
    if (!poly.npoints)
        break;                            /* nothing to do if no verticies */

    hilight = P->pdrawm & DR_HILITE;
    fillcolor = ltop[P->pcolour - MINLCOLOR]; /* logical -> physical color */
    fprop = property(fillcolor, NOGREY, FALSE);

    for (j = 0; j < P->pnvert; j++) {
        poly.pt[j].x = P->pvert[j][0];
        poly.pt[j].y = P->pvert[j][1];
    }

    if (!P->vpnumber) {                  /* pixel poly */
	if (P->pcolour != -1) {
            vectaker = pixvec;
	    drawpoly();
	} else {
            vectaker = pixxvec;
	    drawpoly();
        }
    } else {							/* logical poly */
        i = 0;
        while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
    	if (P->pdrawm & DR_NORDRW)    {      /* logical nondrawing poly */
	   /* only draw in current viewport, first checking to see
           if this is a slave */
           if (VP.master) {           /* if this is a slave */
               /* find the slave vpinfo entry for scaling */
               for (j = 16; j < 31; j++) {
                   if (vpinfo[j].vpnumber == VP.vpnumber) {
                       i = j;
                       break;
                   }
               }
           }
	   scalepoly(i);
	   if (P->pcolour == -1) {
		send("******** XOR LOGICAL POLY NOT DRAWN! ***********");
		break;
	   }
           if (hilight)
               fprop = fprop | (GREY2 << 1);
           vectaker = pixlvec;
	   drawpoly();
	} else {					/* logical drawing poly */
/* add to display list */
		vtran(P->vpnumber);
		appendpoly(i);
/* draw poly in master viewport and any slaves */
		redrawpoly(i);
        }
    }
    break;
#undef PT
#undef VP

case PSYNC:                           /* Synchronise */
case PDIGTIZE:                        /* Process digitizer sample */
case PECHAR:                          /* End character string */
case PTPROMPT:                        /* Position to dual-screen prompt area */
    break;


case PDCURS:                          /* Draw raster cursor */
#define PT pkdcurs
    memcpy((char *)&curp, P->pcaddr, sizeof(struct rcursor));
    fg = 16;
    dcx = P->px - curp.curx;
    dcy = P->py - curp.cury;
    for (i = 0; i < 16; i++) {
        curam[i] = curp.curand[i];
        curxm[i] = curp.curxor[i];
    }
    /* clip if raster overlaps any edge of graphic area */
    if (dcy < 0) {
        dcy = -dcy;
        for (i = 0; i < (16 - dcy); i++) {
            curam[i] = curam[i + dcy];
            curxm[i] = curxm[i + dcy];
        }
        fg -= dcy;                    /* Draw fewer lines */
        dcy = 0;                      /* Start at top of screen */
    } else if ((dcy + fg) > cfr.ysize) {
        fg = cfr.ysize - dcy;
    }
    if (dcx < 0) {
        j = -dcx;
        fpat = 0xFFFF;
        fpat >>= 16 - j;
        for (i = 0; i < fg; i++) {
            curam[i] = (curam[i] << j) | fpat;
            curxm[i] = curxm[i] << j;
        }
        dcx = 0;
    } else if ((dcx + 16) > XSIZE) {
        j = 16 - (XSIZE - dcx);
        fpat = 0xFFFF;
        fpat <<= 16 - j;
        fpat &= 0xFFFF;
        for (i = 0; i < fg; i++) {
            curam[i] = (curam[i] >> j) | fpat;
            curxm[i] = curxm[i] >> j;
        }
        dcx = XSIZE - 16;
    }
    /* Now align the pattern with the words in display RAM */
    j = 8 - (dcx % 8);
    lpat = (0xFFL >> (8 - j)) | (0xFF0000L << j);
    for (i = 0; i < fg; i++) {
        curam[i] = (curam[i] << j) | lpat;
        curxm[i] <<= j;
    }
    egacur(dcx, dcy, fg);
#undef PT
    break;

case PCCURS:
    egaccur();
    break;


case POPENVP:
    openvp(pkt);
    newpkt = TRUE;       
    break;

case PCLOSEVP:
    closevp(pkt);
    break;


case PVIEWPORT:
    pviewport(pkt);
    newpkt = TRUE;
    break;

case PSTRING:
#define PT pkstring
#ifdef SMART
    if (cflag == -1) {                /* no command */
        newpkt = FALSE;
        break;
    } else {
            i = 0;
            while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
            if (vpinfo[i].vpnumber != P->vpnumber)
                break;
            rwidth = vpinfo[i].vrxh - vpinfo[i].vrxl;
            rheight = vpinfo[i].vryh - vpinfo[i].vryl;
            vpinfo[i].vrxl += zptab[cflag].dxl * rwidth / 4;
            vpinfo[i].vryl += zptab[cflag].dyl * rheight / 4;
            vpinfo[i].vrxh += zptab[cflag].dxh * rwidth / 4;
            vpinfo[i].vryh += zptab[cflag].dyh * rheight / 4;

            ctext[0] = 0;
            strcat(ctext,"zoom $0 ");
            gcvt(vpinfo[i].vrxl,8,rbuff);
            strcat(ctext,rbuff);
            strcat(ctext," ");
            gcvt(vpinfo[i].vryl,8,rbuff);
            strcat(ctext,rbuff);
            strcat(ctext," ");
            gcvt(vpinfo[i].vrxh,8,rbuff);
            strcat(ctext,rbuff);
            strcat(ctext," ");
            gcvt(vpinfo[i].vryh,8,rbuff);
            strcat(ctext,rbuff);
            strcat(ctext," ");
            cpoint = ctext;
            P->constr = cpoint;
            cflag = -1;
            newpkt = TRUE;
            break;
    }
#else
    break;
#endif
#undef PT

case PTABCFG:
#define PT pktconfig
    /* this driver doesn't use PTABCFG */
    newpkt = FALSE;
    break;
#undef PT

case PDRAG:
#define PT pkdrag
    newpkt = FALSE;                   /* for most cases */
    i = 0;
    while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
    if (vpinfo[i].vpnumber != P->vpnumber)
        break;                        /* do nothing if viewport not found */
    prop = property(P->ptcolor - MINLCOLOR, NOGREY, TRUE);
    v = vtran(P->vpnumber);           /* init ecurwind */
    hilight = FALSE;                  /* drag vectors are not hilighted */

    if (P->dmode == 0) {              /* drag vector in packet */
        if (vpinfo[i].dstate & 2)  {  /* logical vector */
            slvec(P->pfx, P->pfy, P->ptx,
                  P->pty, prop);
        } else {                      /* absolute pixel vector */
            tadsvec(P->pfx, ybias - P->pfy, P->ptx,
                   ybias - P->pty, prop);
        }
        if (!(vpinfo[i].dstate & 1)) {/* if list hasn't overflowed */
            dappend(P->pfx, P->pfy, P->ptx,
                    P->pty, prop, i);
            vpinfo[i].drs = vpinfo[i].dioffset / 2;
        }
        break;
    }

    if (P->dmode == 1) {              /* initialize dstate 0 */
        vpinfo[i].dstate = 0;         /* set drag state = non-movable */
        vpinfo[i].drs = 0;
        vpinfo[i].dioffset = 0;       /* erase drag list */
        break;
    }

    if (P->dmode == 2) {              /* initialize dstate 2 */
        vpinfo[i].drx = 0;            /* init offset to 0 */
        vpinfo[i].dry = 0;
        vpinfo[i].dstate = 2;         /* set drag state = movable */
        vpinfo[i].drs = 0;
        vpinfo[i].dioffset = 0;       /* erase drag list */
        break;
    }

    if (P->dmode == 3) {              /* try to erase drag image */
        if ((vpinfo[i].dstate & 1) || (vpinfo[i].dioffset == 0)) { 
            /* if list overflowed or is empty */
            P->dmode = 0;
            newpkt = TRUE;
            break;
        }
        dredraw(v,1,0,vpinfo[i].drs,i); /* erase just what's drawn */
        vpinfo[i].dstate &= 0xfb;     /* turn off drag flag */
        vpinfo[i].drs = 0;            /* nothing now drawn! */
        P->dmode = 1;                 /* signal success in erasing */
        newpkt = TRUE;
        break;
    }

    if (P->dmode == 4) {              /* displace drag image */
        if ((vpinfo[i].dstate & 1) || (vpinfo[i].dioffset == 0)) { 
            /* if list overflowed or is empty */
            vpinfo[i].dioffset = 0;
            P->dmode = 0;
            newpkt = TRUE;
            break;
        } else {
            if (vpinfo[i].dstate & 4) { /* if already dragging */
                P->dmode = dredraw(v,0,vpinfo[i].drs,vpinfo[i].dioffset/2,i);
            } else {
                vpinfo[i].drx = P->pfx;
                vpinfo[i].dry = P->pfy; /* displacement */
                vpinfo[i].dstate |= 4;/* set drag flag */
                P->dmode = dredraw(v,0,0,vpinfo[i].dioffset/2,i);
            }
            if (P->dmode == 2)
                vpinfo[i].dstate &= 0xfb; /* turn off drag flag */
            newpkt = TRUE;
            break;
        }
    }

    break;                            /* catch any illegal dmode */
#undef PT

case PLOPEN:
    plopen(pkt);
    break;

case PREVEC:
    newpkt = prevec(pkt);
    break;


default:
    send("\n** Undefined PROPKT function ");
    send(" **\n");
}
else  {                               /* !foreground => background */
    switch(pkt->pfunc)   {

    case PVEC:                        /* add vector to display list
                                      but don't draw it on screen */
#define PT pkvec
        if (P->pcolour == -1) {       /* xor vector */
            prop = property(8, NOGREY, TRUE);
        } else {
            prop = property(ltop[P->pcolour - MINLCOLOR], NOGREY, FALSE);
            if (P->pdrawm & DR_HILITE)
                prop |= property(0, GREY2, FALSE);
        }
        if (!P->vpnumber) {
            /* don't try to draw nondrawing (viewport 0) vectors */
        } else {
            /* viewport specific, logical vectors */
            if (!(P->pdrawm && DR_NORDRW)) /* if not nondrawing */
                /* just add to or remove from display list, don't draw */
                luvec(P->pfx, P->pfy, P->ptx, P->pty,
                      prop, P->vpnumber);
        }    
#undef PT
        break;

    case PFILL:                       /* Fill polygon */
#define PT pkfille
#define VP vpinfo[i]
    poly.npoints = P->pnvert;
    if (!poly.npoints)
        break;                            /* nothing to do if no verticies */

    hilight = P->pdrawm & DR_HILITE;
    fillcolor = ltop[P->pcolour - MINLCOLOR]; /* logical -> physical color */
    fprop = property(fillcolor, NOGREY, FALSE);

    for (j = 0; j < P->pnvert; j++) {
        poly.pt[j].x = P->pvert[j][0];
        poly.pt[j].y = P->pvert[j][1];
    }

    if (!P->vpnumber) {                  /* pixel poly */
    } else {							/* logical poly */
        i = 0;
        while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
    	if (P->pdrawm & DR_NORDRW)    {      /* logical nondrawing poly */
	} else {					/* logical drawing poly */
/* add to display list */
		appendpoly(i);
        }
    }
    break;
#undef PT
#undef VP

    case PHLITE:
    case PDHLITE:
#define PT pkhlite
        P->pstat = FALSE;
        newpkt = TRUE;
        break;
#undef PT

    case PCMARK:
    case PMARK:
#define PT pkmark
        P->pstat = FAIL;
        newpkt = TRUE;
        break;
#undef PT

    case PREDRAW:
#define PT pkredraw
        P->pstat = 1;
        newpkt = TRUE;
        break;
#undef PT

    case PQPLOT:
#define PT pkqplot
        P->pstat = FAIL;
        newpkt = TRUE;
        break;
#undef PT

    case PRPEN:
        break;

    case PCOMMAND:
#define PT pkcommand
        P->pstat = FALSE;
        newpkt = TRUE;
        break;
#undef PT

    case PXPCOMD:
#define PT pkxpcomd
        P->pstat = newpkt = TRUE;
        break;
#undef PT

    case PGOGRAPH:
        prevscr = GRAPHSCR;
        break;

    case PGOTEXT:
    case PGOTEXTU:
        prevscr = TEXTSCR;
        break;

    case PBSHELL:
    case PASHELL:
#define PT pkshell
        backflip(P->status);

        switch(P->status)   {

        case 0:                       /* from bg -> fg */
            foreground = 1;
            if (prevscr == GRAPHSCR)
                P->repaint = RP_STAT | RP_MENU | RP_SCROLL | RP_BS;
            else
                P->repaint = RP_TEXT;
            newpkt = TRUE;
            break;

        case 1:                       /* regular shell command */
            foreground = 1;
            break;

        case 2:
        case 3:
            foreground = 0;
            break;

        default:
            break;
        }
#undef PT
        break;

    case POPENVP:
        openvp(pkt);
        newpkt = TRUE;       
        break;

    case PCLOSEVP:
        closevp(pkt);
        break;


    case PVIEWPORT:
        pviewport(pkt);
        newpkt = TRUE;
        break;


    case PLOPEN:
        plopen(pkt);
        break;

    case PREVEC:
        newpkt = prevec(pkt);
        break;


    default:
        break;
    }
}   

#ifdef trace
if (tracing && newpkt)
  dumppkt('*',pkt);
  if (tracing && logfile)
      logchar((char)'\n');
#endif
  if (overflow && foreground)  {
	send("\n display list overflow \n");
	switch(overflow) {
		case 1:
			break;
		case 2:
			send("wtran failed\n");
for (j = 0; j < 16; j++) {
    send("j : wtranv[j]:");
    senddec(j);
    send(",");
    senddec(wtranv[j]);
    send("\n");
}
send("ecurwind =");
sendhex(4,ecurwind);
send(" targetv=");
senddec(targetv);
send(" chreg:");
senddec(chreg);
send("\n");
			break;
		case 3:
			send("svtran failed\n");
			break;
		default:
			break;
  	}
	overflow = 0;
  }
  return newpkt;
  }

  /*===================================================*/
    
vpidump()
{
    short i,j;
    short *fetch;

    /* and for testing, dump vpinfo for our viewing pleasure */
    for (i = 0; i < 31; i++) {
        if (vpinfo[i].vpnumber) {
            send("\n");
            senddec(i);
            send("=");
            senddec(vpinfo[i].vpnumber);
            send(" ");
            senddec(vpinfo[i].vpxmin);
            send(" ");          
            senddec(vpinfo[i].vpymin);
            send(" ");
            senddec(vpinfo[i].vpxmax);
            send(" ");
            senddec(vpinfo[i].vpymax);
            send(" ");
            senddec(vpinfo[i].lgxl);
            send(" ");
            senddec(vpinfo[i].lgyl);
            send(" ");
            senddec(vpinfo[i].lgxh);
            send(" ");
            senddec(vpinfo[i].lgyh);
            send(" ");
            senddec(vpinfo[i].wixdots);
            send(" ");
            senddec(vpinfo[i].wiydots);
            send(" ");
            senddec(vpinfo[i].lgxdots);
            send(" ");
            senddec(vpinfo[i].lgydots);
            send(" ");
            senddec(vpinfo[i].dstate);
            send(" ");
            senddec(vpinfo[i].vxdots);
            send(" ");
            senddec(vpinfo[i].vydots);
            send(" ");
            senddec(vpinfo[i].drs);
            send(" ");
            senddec(vpinfo[i].projid);
            send(" ");
            senddec(vpinfo[i].master);
            send(" ");
            senddec(vpinfo[i].slaves);
            send(" ");
            senddec(vpinfo[i].vectors);
            send(" ");
            senddec(vpinfo[i].ioffset);
        }
    }
    send("\n ecurwind:");
    sendlong(ecurwind);
    send("\n");

    for (j = 0; j < 16; j++)   {
        if (vpinfo[j].vpnumber && vpinfo[i].dioffset) {
            send("viewport ");
            senddec(j);
            send("\n");
            for (i = 0; i < vpinfo[j].dioffset; i += 2) {
                fetch = (short *)vpinfo[j].ddlist;
                senddec(*(fetch + i));
                send(" ");
            }
        }
    }
}

static void
/*FCN*/fdraw(curx,cury,x,y,color,hilight,bs_slave)
  short curx,cury,x,y,color,hilight,bs_slave;
{
    short prop;
#ifdef serial
fprintf(sterm,"x:%x,y:%x,tx:%x,ty:%x\n",curx,cury,x,y);
#endif
    prop = property(ltop[color - MINLCOLOR],NOGREY, FALSE);
    if (foreground)
        lvec(curx, cury, x, y, prop, bs_slave);
    else
        luvec(curx, cury, x, y, prop, bs_slave);     
}


static void
/*FCN*/openvp(pkt)
  struct pkinit *pkt;
{
    short i;

#define PT pkovp
    i = 0;
    while(vpinfo[i].vpnumber && i < 16) i++; /* find an empty vpinfo slot */
    vpinfo[i].dlist = (char *)malloc(DLSIZE) + 2;
    if (vpinfo[i].dlist == (char *)2) {
	send("malloc error viewport i:");
	senddec(i);
	send("\n");
        return;
    }
    vpinfo[i].ioffset = vpinfo[i].dioffset = 0;
    vpinfo[i].ddlist = (char *)malloc(DDLSIZE);
    vpinfo[i].vpnumber = P->vpnumber;
    vpinfo[i].vpxmin = P->vpxmin;
    vpinfo[i].vpymin = P->vpymin;
    vpinfo[i].vpxmax = P->vpxmax;
    vpinfo[i].vpymax = P->vpymax;
    vpinfo[i].wixdots = P->vpxmax - P->vpxmin;
    vpinfo[i].wiydots = P->vpymax - P->vpymin;
    vpinfo[i].master = vpinfo[i].slaves = vpinfo[i].vectors = 0;
    /* your logical space should be scaled to the same proportions
       as the viewport it will be displayed in, acad sends you
       P->lgxdots, P->lgydots as the largest allowable values,
       you scale down from there if necessary. Our driver
       just uses these default maximum values. */
    vpinfo[i].lgxdots = P->gxdots;
    vpinfo[i].lgydots = P->gydots;
#undef PT
}

static void
/*FCN*/closevp(pkt)
  struct pkinit *pkt;
{
    short i, j;
#define PT pkcvp
    i = 0;
    while((vpinfo[i].vpnumber != P->vpnumber) && (i < 16)) i++;
    if (vpinfo[i].vpnumber != P->vpnumber)
        return;                       /* do nothing if viewport not found */
    /* was this a slave viewport? */
    if (vpinfo[i].master > 0) {
        /* subtract a slave */
        if (vpinfo[((vtran(vpinfo[i].master) >> 8) - 1)].slaves > 0)
            vpinfo[((vtran(vpinfo[i].master) >> 8) - 1)].slaves -= 1;
        /* now zap slave vpinfo */
        vpinfo[(svtran(P->vpnumber) - 1)].vpnumber = 0;
        vpinfo[i].master = 0;
    }
    free(vpinfo[i].dlist - 2);
    free(vpinfo[i].ddlist);

    /* now check to see if any vp was slaved to i! */
    for (j = 0; j < 16; j++) {
        if ((i != j) && (vpinfo[j].master == vpinfo[i].vpnumber) &&
            (vpinfo[j].vpnumber != 0)) {
            emancipate(j);
        }
    }

    vpinfo[i].vpnumber = 0;           /* 0 vpnumber => closed */
    vpinfo[i].ioffset = vpinfo[i].dioffset = 0;
#undef PT
}

static void
/*FCN*/pviewport(pkt)
  struct pkinit *pkt;
{
    short i, j, v;
#define PT pkview
    i = 0;
    while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
    if (vpinfo[i].vpnumber != P->vpnumber)
        return;                       /* do nothing if viewport not found */
    vpinfo[i].lgxl = P->gxl;
    vpinfo[i].lgyl = P->gyl;
    vpinfo[i].lgxh = P->gxh;
    vpinfo[i].lgyh = P->gyh;
    vpinfo[i].vxdots = vpinfo[i].lgxh - vpinfo[i].lgxl;
    vpinfo[i].vydots = vpinfo[i].lgyh - vpinfo[i].lgyl;
    /* figure viewport location in projection space to ease
        calculations for local pan & zoom commands */
#define _vrxl vpinfo[i].vrxl
#define _vryl vpinfo[i].vryl
#define _vrxh vpinfo[i].vrxh
#define _vryh vpinfo[i].vryh
#define _rxl vpinfo[i].rxl
#define _ryl vpinfo[i].ryl
#define _rxh vpinfo[i].rxh
#define _ryh vpinfo[i].ryh
#define _lgxl vpinfo[i].lgxl
#define _lgyl vpinfo[i].lgyl
#define _lgxh vpinfo[i].lgxh
#define _lgyh vpinfo[i].lgyh
#define _lgxdots vpinfo[i].lgxdots
#define _lgydots vpinfo[i].lgydots
    _vrxl = _rxl + (_rxh - _rxl) * (_lgxl) / (_lgxdots);
    _vryl = _ryl + (_ryh - _ryl) * (_lgyl) / (_lgydots);
    _vrxh = _rxl + (_rxh - _rxl) * (_lgxh) / (_lgxdots);
    _vryh = _ryl + (_ryh - _ryl) * (_lgyh) / (_lgydots);
#undef _vrxl
#undef _vryl
#undef _vrxh
#undef _vryh
#undef _rxl 
#undef _ryl 
#undef _rxh
#undef _ryh
#undef _lgxl
#undef _lgyl
#undef _lgxh
#undef _lgyh
#undef _lgxdots
#undef _lgydots

    P->master = vpinfo[i].vpnumber;   /* default => it is its own master */

/* while DEBUGGING */
/*	P->vflags = NO_BS;
	return;
*/

    /* if this is was a slave vp, see if it still should be */
    if (vpinfo[i].master > 0) {       /* already a slave ? */
        j = (vtran(vpinfo[i].master) >> 8)  - 1;
        if ((stest(i,j)) && (vpinfo[j].master == 0)) {
            /* still a subset      of master ? */
            if (scale(i,j,v = (svtran(P->vpnumber) - 1))) {
                P->vflags = NO_VEC;   /* continue slave status */
                P->master = vpinfo[i].master;
                return;               /* skip the rest of processing */
            }
        }
        /* no longer subset, discontinue slave status! */
        /* subtract a        slave */
        if (vpinfo[j].slaves > 0)
            vpinfo[j].slaves -= 1;
        /* now zap slave vpinfo */
        vpinfo[(svtran(P->vpnumber) - 1)].vpnumber = 0;
        vpinfo[i].master = 0;         /* free the slave */
        vpinfo[i].vectors = 1;        /* request vectors */
        P->vflags = 0;
    }

    /* now check to see if any vp was slaved to i! and
    if it still should be */
    if (vpinfo[i].slaves > 0) {
        for (j = 0; j < 16; j++) {    /* scan 'em all */
            if ((i != j) && (vpinfo[j].master == vpinfo[i].vpnumber) &&
                (vpinfo[j].vpnumber != 0) && !stest(i,j)) {
                emancipate(j);        /* free it if we are not master */
                if (vpinfo[i].slaves > 0)
                    vpinfo[i].slaves -= 1; /* subtract one slave */
            }
            if (vpinfo[j].master == vpinfo[i].vpnumber) {
                /* update and check slave's scaling factors */
                if (!scale(j,i,v = svtran(vpinfo[j].vpnumber) - 1)) {
                    emancipate(j);
                    if (vpinfo[i].slaves > 0)
                        vpinfo[i].slaves -= 1;
                }
            }
        }
    }

    /* now check to see if this is a subset of another vp 
    (which is not a slave, to avoid thrashing)
    and if we are not already someone's slave:
    i.e., should we become a slave? While we are at it,
    check to see if the new master needs vectors... */
    if ((j = viewck(i)) && (vpinfo[i].master == 0)) {
        j--;
        v = 0;
        if (vpinfo[j].master == 0) {  /* only if J is not a slave! */
            if (vpinfo[i].slaves)     /* if we were a master, remember */
                v++;
            if (slave(i,j)) {         /* set up slave vpinfo struct */
                v++;
                vpinfo[i].master = vpinfo[j].vpnumber;
                P->vflags = NO_VEC;
                P->master = vpinfo[i].master;
                vpinfo[i].ioffset = 0;/* kill vectors in list */
            }
            if (v == 2) {
                /* if we were a master and are being enslaved, transfer our
                slaves to the new master or free them */
                transfer(j,i);
            }
            if (v) {                  /* if we became a slave */
                return;               /* don't bother to process more */
            }
        }
    }

    /* has this vp just become a master of one or more slaves? */
    for (j = 0; j < 16; j++) {
        v = 0;
        if (i != j && (stest(j,i)) && vpinfo[j].master == 0) {
            /* not already a slave, but could be ours */
            if (vpinfo[j].slaves)      { /* if we may be enslaving a master */
                v++;                  /* remember it for a while */
            }           
            if (slave(j,i)) {
                v++;                  /* yes, it is becoming a slave */
                vpinfo[j].master = vpinfo[i].vpnumber;
                vpinfo[j].ioffset = 0;
                vpinfo[j].vectors = -1; /* flag for prevec! */
            }
            if (v == 2) {
                /* transfer any of former master's slaves to this new master */
                transfer(i,j);
            }                   
        }
    }
#undef PT
}

static void
/*FCN*/plopen(pkt)
  struct pkinit *pkt;
{
    short i;
#define PT pklopen
    i = 0;
    while(vpinfo[i].vpnumber != P->vpnumber && i < 16) i++;
    if (vpinfo[i].vpnumber != P->vpnumber)
        return;                       /* do nothing if viewport not found */

    vpinfo[i].projid = P->projid;
    vpinfo[i].rxl = P->rxl;
    vpinfo[i].ryl = P->ryl;
    vpinfo[i].rxh = P->rxh;
    vpinfo[i].ryh = P->ryh;
#undef PT
}

static short 
/*FCN*/prevec(pkt)
  struct pkinit *pkt;
{
    short i;
#define PT pkrvec
    for (i = 0; i < 16; i++) {
        /* see if any viewports need their vectors turned off */
        if (vpinfo[i].vectors == -1) {
            P->vpnumber = vpinfo[i].vpnumber;
            P->rvflags = UN_VEC | AGAIN;
            vpinfo[i].vectors = 0;
            P->master = vpinfo[i].master;
            return TRUE;
        }
        /* see if any viewports need their vectors turned back on */
        if (vpinfo[i].vectors & 2) {
            P->vpnumber = vpinfo[i].vpnumber;
            P->rvflags = YES_VEC | AGAIN;
            vpinfo[i].vectors &= 1;
            P->master = vpinfo[i].vpnumber;
            return TRUE;
        }
    }
    return FALSE;
#undef PT
}
/*=======================================================*/

/* we are enslaving a master. Scan all viewports to find the
former master's slaves and transfer them to the new master
if they qualify as legal slaves for it (they might not
if zoomed in too far).
j = old master
i = new master
*/
static void
/*FCN*/transfer(i,j)
  short i,j;
{
    short v;

    for (v = 0; v < 16; v++) {
        if (vpinfo[v].master == vpinfo[j].vpnumber) {
            emancipate(v);
            if (slave(v,i)) {         /* make sure they are legal slaves */
                vpinfo[v].master = vpinfo[i].vpnumber;
                vpinfo[v].vectors = -1; /* tell PREVEC to turn off vectors*/
                /* yeah, I know they are off but we also have to tell acad
                the identity of the slave's new master! */
            }
        }
    }
}

/* free up a slave */
static void
/*FCN*/emancipate(j)
  short j;
{
    vpinfo[j].vectors = 3;            /* flag for PREVEC to YES_VEC 
                                      and then request vectors */
    vpinfo[j].master = 0;
    /* zap slave vpinfo structure */
    vpinfo[(svtran(vpinfo[j].vpnumber) - 1)].vpnumber = 0;
}


/* establish a slave vpinfo structure k for viewport i
slaved to master viewport j. We initialize just the
data elements needed by iclip!  The purpose of storing
these computed values is to speed fastdraw & fredraw using iclip.
After computing, check to be sure we have more logical than
physical pixels in each direction. If not, we are zoomed in too
far and should not allow this to be a slave, so we return 0 to
block enslavement and cancel the slave vpinfo setup */
static short
/*FCN*/slave(i,j)
  short i, j;
{
    short k;


    for (k = 16; k < 31; k++)
        if (vpinfo[k].vpnumber == 0) break; /* find 1st unused slot */
    vpinfo[k].vpnumber = vpinfo[i].vpnumber; /* actual vp number */
    vpinfo[k].vpxmin = vpinfo[i].vpxmin;
    vpinfo[k].vpymin = vpinfo[i].vpymin;
    vpinfo[k].wixdots = vpinfo[i].wixdots;
    vpinfo[k].wiydots = vpinfo[i].wiydots;
    vpinfo[k].master = vpinfo[j].vpnumber;

    if (scale(i,j,k)) {               /* figure viewport scale factors */
        svtran(vpinfo[i].vpnumber);   /* figure ecurwind */
        vpinfo[j].slaves +=      1;   /* add one slave to the master */
        vpinfo[i].slaves = 0;         /* clean slave */
        vpinfo[i].vectors =      0;
        return 1;                     /* report successful enslavement */
    } else {
        vpinfo[k].vpnumber = 0;       /* cancel slave vpinfo */
        return 0;                     /* report zoomed in too far */
    }
}


/* figure slave vpinfo scale factors and be sure that
the number of physical pixels in each direction of
the potential slave's viewport smaller than the number
of logical pixels in the viewport in the master's logical space.
Return 1 if OK, return 0 if not usable as a slave.
j = master
i = slave
k = slave scaling vpinfo
*/
static short 
/*FCN*/scale(i,j,k)
  short i,j,k;
{
    float temp;
    short h;

    rwidth = vpinfo[j].rxh - vpinfo[j].rxl;
    rheight = vpinfo[j].ryh - vpinfo[j].ryl;


    temp = (vpinfo[i].vrxl - vpinfo[j].rxl) * vpinfo[j].lgxdots / rwidth;
    gcvt(temp,8,ctext);
    for (h = 0; h < strlen(ctext); h++)
        if (ctext[h] == '.') ctext[h] = 0;
    vpinfo[k].lgxl = atol(ctext); 

    temp = (vpinfo[i].vrxh - vpinfo[j].rxl) * vpinfo[j].lgxdots / rwidth;
    gcvt(temp,8,ctext);
    for (h = 0; h < strlen(ctext); h++)
        if (ctext[h] == '.') ctext[h] = 0; 
    vpinfo[k].lgxh = atol(ctext);

    temp = (vpinfo[i].vryl -vpinfo[j].ryl) * vpinfo[j].lgydots / rheight;
    gcvt(temp,8,ctext);
    for (h = 0; h < strlen(ctext); h++)
        if (ctext[h] == '.') ctext[h] = 0;
    vpinfo[k].lgyl = atol(ctext);

    temp = (vpinfo[i].vryh -vpinfo[j].ryl) * vpinfo[j].lgydots / rheight;
    gcvt(temp,8,ctext);
    for (h = 0; h < strlen(ctext); h++)
        if (ctext[h] == '.') ctext[h] = 0;
    vpinfo[k].lgyh = atol(ctext);

    vpinfo[k].vxdots = vpinfo[k].lgxh - vpinfo[k].lgxl;
    vpinfo[k].vydots = vpinfo[k].lgyh - vpinfo[k].lgyl;

    if ((vpinfo[k].wixdots < vpinfo[k].vxdots) &&
        (vpinfo[k].wiydots < vpinfo[k].vydots))
        return 1;
    else
        return 0;
}


/* is viewport "i" a proper subset of another which is free?
If so, return the number of the master vp */
static short 
/*FCN*/viewck(i)
  short i;
{
    short j, k;
    for (j = 0; j < 16; j++) {
        if ((k = stest(i,j)) && (vpinfo[j].master == 0)) {
            return k;
        }
    }
    return 0;                         /* no superset found */
}

/* is viewport i a proper subset of viewport j? 
return 0 if not, or j+1 if it is. i and j are
indexes into vpinfo[] (not raw viewport numbers) */
static short
/*FCN*/stest(i,j)
  short i, j;
{

    if ((i != j) && (vpinfo[j].vpnumber != 0) &&
        (vpinfo[i].vpnumber != 0) &&
        (vpinfo[i].projid == vpinfo[j].projid) &&
        (vpinfo[i].vrxl > vpinfo[j].rxl) &&
        (vpinfo[i].vrxh < vpinfo[j].rxh) &&
        (vpinfo[i].vryl > vpinfo[j].ryl) &&
        (vpinfo[i].vryh < vpinfo[j].ryh)) { 
        return j + 1;                 /* yes, i is subset of j */
    } else {
        return 0;
    }
}


/* DREDRAW from drag display list:
if nonstop = 1, do entire list (we are erasing)
if nonstop = 0  (we are redrawing a moved drag image) 
 - do at most 10 vector sequences at a time, then if not done
 with the list, return to give acad a chance to poll user input
In any case, start at "first" and stop at "last"

 Return = 1 if not done with list, = 2 if done.
*/
static short 
/*FCN*/dredraw(tport, nonstop, first, last, i)
  short tport, nonstop, first, last, i;
{
    short *fetch;
    short p, j, x1, y1, x2;
    unsigned short y2;
    short drawn, dlim, xo, yo, prop;

    fetch = (short *)vpinfo[i].ddlist;
    drawn = 0;
    xo = vpinfo[i].drx;
    yo = vpinfo[i].dry;
    if (nonstop)
        dlim = 0x7fff;                /* do every vector non-stop */
    else
        dlim = 10;                    /* stop every 10 sequences */

    p = first;                        /* start where start from */
    do  {
        j = 0;
        x1 = *(fetch + p + j++);
        y1 = *(fetch + p + j++);
        prop = *(fetch + p + j++);
        j++;                          /* skip 0xffff flag */
        for (; ((p + j) < last);)  {
            x2 = *(fetch + p + j++);
            y2 = *(fetch + p + j++);
            if (y2 == 0xffff) {
                j -= 4;               /* back up to start of new sequence */
                break;
            }

            if (!(x2 & 0x8000))  {
                if (vpinfo[i].dstate & 2)
                    slvec((x1 & 0x7fff) + xo, /* apply move offset */
                          (y1 & 0x7fff) + yo, (x2 & 0x7fff) + xo,
                          (y2 & 0x7fff) + yo, prop);
                /* draw any unerased logical vectors */
                else
                    tadsvec(x1 & 0x7fff,
                           ybias - y1 & 0x7fff, x2 & 0x7fff,
                           ybias - y2 & 0x7fff, prop);
                /* draw any unerased absolute vectors */
            }
            x1 = x2;
            y1 = y2;
        }
        p += j;
        /* now we've done one sequence of vectors */
        /* count # sequences here, return after 10 if not done */
    } while (((p + 4) < last) && (drawn++ < dlim));

    vpinfo[i].drs = p;                /* save pointer into drag list */
    if ((p + 4) < last)
        return 1;                     /* signal not done yet */
    else
        return 2;                     /* signal all done! */
} 


/* DAPPEND - add a vector to the drag display list */
/* get here if we need to append and list isn't full */
static short
/*FCN*/dappend(x1, y1, x2, y2, prop, i)
  short x1, x2, y1, y2, prop, i;
{
    short j;
    unsigned short t1, t2;
    short *fetch;

    fetch = (short *)(vpinfo[i].ddlist + (long)vpinfo[i].dioffset);
    /* point to end of drag list */

    if (vpinfo[i].dioffset > DDLSIZE-12) {
        send("\n Drag display list overflow!\n");
        vpinfo[i].dstate |= 1;        /* set overflow flag */
        return 0;
    }
    if (vpinfo[i].dioffset > 8)   {
        if ((y1 == *(fetch - 1)) && (x1 == *(fetch - 2))) {
            for (j = 3; j < (vpinfo[i].dioffset / 2);)  {
                t2 = *(fetch - j++);
                t1 = *(fetch - j++);
                if (t2 == 0xffff)
                    break;
            }
            if (t1 == prop)   {
                *(fetch++) = x2;
                *(fetch++) = y2;
                vpinfo[i].dioffset += 4;
                return 0;
            }
        }
    }
    /* fall to here if fresh entry required */
    *(fetch++) = x1 | 0x8000;
    *(fetch++) = y1;
    *(fetch++) = prop;
    *(fetch++) = 0xffff;
    *(fetch++) = x2;
    *(fetch++) = y2;
    vpinfo[i].dioffset += 12;
    return 0;
}


/* REDRAW from stored display list:
   possible pairs to fetch from list
   x1, y1 at start of list
   x1, y1 at start of vector sequence
   prop, ffff
   x2,y2 following
   x2,y2 with moveonly set (erased vectors end in moveonly)

   vport = i as used in vpinfo[i], i.e., numbered from 0...15

*/
static void
/*FCN*/redraw(vport)
  short vport;
{
    short *fetch;
    short  x1, y1, x2, k;
    unsigned short y2;
    short prop;
    int words, i, j;
    long esavewind;


    esavewind = ecurwind;           /* redrawpoly will trash ecurwind */

    fetch = (short *)vpinfo[vport].dlist;
    if (!(words = (vpinfo[vport].ioffset)/2))
        return;                     /* don't try to draw if empty list */

    i = 0;
    do  {
        j = 0;
        x1 = *(fetch + i + j++);
        y1 = *(fetch + i + j++);
        prop = *(fetch + i + j++);   
        if (prop & 0xf000) {          /* found polygon */
            poly.npoints = (prop & 0xf000) >> 12;
            poly.pt[0].x = x1 & 0x7fff;
            poly.pt[0].y = y1 & 0x7fff;
            fprop = prop & 0xff;
            j++;
            for (k = 1; k < poly.npoints; k++) {
                poly.pt[k].x = *(fetch + i + j++);
                poly.pt[k].y = *(fetch + i + j++);
            }
            redrawpoly(vport);
            j += 4;
        } else {
            j += 1;                       /* skip 0xffff flag */
            for (; ((i + j) < words);)  {
                x2 = *(fetch + i + j++);
                y2 = *(fetch + i + j++);
                if (y2 == 0xffff) break;
                if (!(x2 & 0x8000))  {
                    slvec(x1 & 0x7fff,
                          y1 & 0x7fff, x2 & 0x7fff,
                          y2 & 0x7fff, prop); /* draw any unerased vectors */
                }

                x1 = x2;
                y1 = y2;
            }
        }
        ecurwind = esavewind;
        /* now we've done one sequence of vectors */
        i += j - 4;                   /* back up to just before 0xffff */
    } while (i + 4 < words);
} 


/*  Put the screen in graphics mode. */
static void
/*FCN*/gograph()
{
	
    nextfree = pushed = 0;            /* local storage pointer & box
                                     pushed count init */

    if (giflag) {                     /* Initial call? */
        ioctl(1, MODESWITCH | cfr.gmode, (char *)0);/* graphic mode*/
        tadsglyph(0, 0, XSIZE, cfr.ysize, 0x68, NOGL);
        tadsglyph(0, topy, ixdots, ybias - topy + 1, 0x78, NOGL);

        if (!notxt)                   /* Graphics text configured off? */
            clearsplit();             /* Clear text area on graphics screen */
    }
    else {
        ioctl(1, MODESWITCH | cfr.gmode, (char *)0);/* graphic mode */
        dglyph = vidbuf;
        sglyph = segsav[0];
        tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x01, 0); /* plane 0 */
        sglyph += (XSIZE * cfr.ysize) / 8;    /* bytes in one bit plane */
        tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x02, 1); /* plane 1 */
        sglyph = segsav[1];
        tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x04, 2); /* plane 2 */
        sglyph += (XSIZE * cfr.ysize) / 8;
        tbyteblt(dglyph, sglyph, 80, cfr.ysize + 1, 0, 0, 0x08, 3); /* plane 3 */
        if (!notxt)                   /* Graphics text configured off? */
            clearsplit();             /* Clear text area on graphics screen */
    }
    initpal();
    prevscr = GRAPHSCR;
}

static void
/*FCN*/wrsplit(c)
  char c;
{
#ifdef flog
    logchar(c);
#endif
#ifdef serial
    fputc(c,sterm);
#endif
    if (foreground && prevscr == GRAPHSCR)
        wrsplit1(c);
}


/*  Write a character to the graphics screen text area. */
static void
/*FCN*/wrsplit1(c)
  unsigned char c;
{
    if (notxt)                        /* If graphics text configured off, */
        return;                       /*   do nothing. */
    switch (c)  {
    case '\r':                        /* Carriage return */
        splitchar = 0;
        break;
    case '\n':                        /* Line feed. */
        if (splitclean)
            break;
        splitchar = 0;
        if (++splitline < cfr.ysize / cfr.charhgt)
            break;
        splitline--;                  /* Advance to next line */
        splitdirty = FALSE;
        dglyph = vidbuf + ((cfr.ysize - SPLITHGT + 1) * BPR);
        sglyph = dglyph + (CHARHGT * BPR);
        tbyteblt(dglyph, sglyph, CHARHGT * BPR, SPLIT - 1, 0, 0, 15, -1);
        tadsglyph(0, cfr.ysize - CHARHGT, CHARWID * TEXTW, CHARHGT, 0x68, NOGL);
        break;
    case '\b':                        /* Backspace. */
        if (splitchar > 0) {
            if (--splitchar == 0)
                splitdirty = FALSE;
            tadsglyph(splitchar * CHARWID, splitline * CHARHGT, CHARWID,
                     CHARHGT, 0x68, NOGL);
        }
        break;
    case '\t':                        /* Tab (does better than DOS!) */
        do
            wrsplit1(' ');
        while (splitchar & 7);
        break;
    default:                          /* Normal character. */
        if (c < ' ' || c > 128)
            break;
        if (splitchar >= TEXTW)       /* If off end of line, */
            wrsplit1('\n');           /*  do a line feed. */
        if (splitchar == 0 && splitdirty) {
            tadsglyph(0, splitline * CHARHGT, BPR * CHARWID, CHARHGT, 0x68, NOGL);
        }
        if (c != ' ') {
             sglyph = (chartab + (c * CHARHGT));
            tadsglyph(splitchar * CHARWID, splitline * CHARHGT, CHARWID,
                     CHARHGT, 0x38, sglyph);
        }
        splitchar++;                  /* Update cursor. */
        splitdirty = TRUE;            /* Non-blank line. */
    }
}


/*  Clear the graphics text area and reset its insertion cursor. */
static
/*FCN*/clearsplit()
{
    tadsglyph(0, cfr.ysize - SPLITHGT, 
             CHARWID * TEXTW, SPLITHGT, 0x68, NOGL);
    splitchar = 0;
    splitline = YSPLIT / CHARHGT;
    splitdirty = FALSE;
}


/* Send a text string to the console--we avoid using the compiler's
   library (e.g., printf) to keep down the size of the driver--
   after all, it is permanently resident. */
static void
/*FCN*/send(s)
  char    *s;
{
    int c;

    while (c = *s++ & 0xFF) {
        if (c == '\n')
            wrsplit('\r');
        wrsplit(c);
    }
}




/*  Calculate the width in bytes of a horizontal segment */
static unsigned
/*FCN*/bytesfor(x, xlen)
  unsigned x, xlen;
{
    unsigned    byte1, byten;

    byte1 = x >> 3;
    byten = (x + xlen -1) >> 3;
    return (byten - byte1 + 1);
}


/*  Calculate the buffer byte offset of the byte containing a point */
static unsigned
/*FCN*/byteof(x, y)
  unsigned      x, y;
{
    return ((x >> 3) + y * BPR);
}


/*  DVTEXT  --  Write text onto screen  */
static void 
/*FCN*/dvtext(x, y, attr, len, s, bg, fg)
  short   x, y, attr, len, bg, fg;
  unsigned char *s;
{
    static char check[] = {0x00, 0x18, 0x00, 0x30, 0x00, 0x60,
                 0x00, 0xC0, 0x01, 0x80, 0xC3, 0x00,
                 0x66, 0x00, 0x3C, 0x00, 0x18, 0x00};
    unsigned    i, c, nx;

    if (len == 0)
        if ((len = strlen(s)) == 0)
            return;
    fg = property(ltop[fg - MINLCOLOR], NOGREY, FALSE);
    bg = property(ltop[bg - MINLCOLOR], NOGREY, FALSE);
    if (attr & TAINV) {
        i = fg;                       /* switch foreground and background */
        fg = bg;
        bg = i;
    }
    if (attr & TAGREY)
        fg |= property(0, GREY1, FALSE);
    nx = (x *= CHARWID);
    y *= CHARHGT;
    tadsglyph(x, y, len * CHARWID, CHARHGT, bg, NOGL);
    while (len-- && (c = *s++) != '\0') {
        if (c == 128) {               /* Check-mark */
            tadsglyph(nx, y + 2, 13, 9, fg, check);
        } else {
            sglyph = (chartab + (c * CHARHGT));
            tadsglyph(nx, y, CHARWID, CHARHGT, fg, sglyph);
        }
        nx += CHARWID;
    }
}


/*  DVLINE  --  Draw line in natural coordinates  */
static void
/*FCN*/dvline(fx, fy, tx, ty, clr)
  short fx, fy, tx, ty;
  short   clr;
{
    if (clr == -1) {
        prop = property(0, NOGREY, TRUE);
    } else {
        prop = property(ltop[clr - MINLCOLOR], NOGREY, FALSE);
    }
    tadsvec(fx, fy, tx, ty, prop);
}


static void
/*FCN*/backflip(status)
  short status;
{

    switch(status) {

    case 0:                           /* returning from bkgnd */
        if (prevscr == GRAPHSCR) {
            restgr();
        } else {

            ioctl(1, MODESWITCH | cfr.tmode, (char *)0);/* text mode */
            cleartext();
        }
        break;

    default:
        break;
    }
}

/* restore & erase graphic screen */
/*FCN*/restgr()
{            
    ioctl(1, MODESWITCH | cfr.gmode, (char *)0);/* graphic mode */
    initpal();
    /* clear entire screen, set up border */
    tadsglyph(0, 0, XSIZE, cfr.ysize, 0x68, NOGL);
    tadsglyph(0, topy, ixdots, ybias - topy + 1, 0x78, NOGL);
    /*  Draw the borders */
    if (!notxt) {
        tadsvec(0, ybias + 1, XRIGHT, ybias + 1, 0x40);
    }
    if (!nomnu) {
        tadsvec(ixdots + 1, ybias, ixdots + 1, 0, 0x40);
    }
    if (!notxt) {
        clearsplit();                 /* clear scrolling text area */
    }
}

/* erase the text screen */
/*FCN*/cleartext()
{
  puts("\x9b2J\0");

}

/* ===================== debugging code added =============== */

/* Display a hex number "v" with "n" digits */
static void
/*FCN*/sendhex(n, v)
  short             n;
  unsigned short    v;
{
    char        s[20];
    int         i;

    for (i = n; i--; v >>= 4)
        s[i] = ((v & 15) >= 10 ? (v & 15) - 10 + 'A' : (v & 15) + '0');
    s[n] = '\0';
    send(s);
}

/* Display a decimal value */
static void
/*FCN*/senddec(v)
  short     v;
{
    sendlong((long)v);
}

/* Display a long decimal value */
static void
/*FCN*/sendlong(v)
  long     v;
{
    if (v < 0) {
        wrsplit('-');
        v = - v;
    }
    if (v >= 10)
        sendlong(v / 10);
    wrsplit('0' + (int)(v % 10));
}


#ifdef trace

/* Special function to dump the contents of a packet for debugging and
   general illucidation of ADI developers.  One would normally not
   include such a routine in the production version of a driver.  Tracing
   can be enabled by hitting ctl-L while in AutoCAD's drawing editor. */
static void
/*FCN*/dumppkt(sentinel,pkt)
  short     sentinel;
  struct pkinit *pkt;
{
    char        *dptr, *iptr;
    short         i, j, count, radix, len;

#define UCHAR(x)        ((x) * 4 + 0)
#define SHORT(x)        ((x) * 4 + 1)
#define ADDRESS(x)      ((x) * 4 + 2)
#define LONG(x)         ((x) * 4 + 3)
#define DOUBLE(x)      -((x) * 4 + 0)
    static char codes[] = {
                 0,
#define I0      1
                 SHORT(1), -SHORT(1), SHORT(7), -SHORT(1), SHORT(2),
                 SHORT(2), UCHAR(2), -LONG(1), SHORT(7), LONG(1),
                 LONG(1), SHORT(1), 0,
#define B0      I0 + 13
                 SHORT(5), LONG(3), 0,
#define T0      B0 + 3
                 SHORT(2), -SHORT(1), LONG(1), SHORT(3), 0,
#define F0      T0 + 5
                 SHORT(1), -SHORT(1), SHORT(26), 0,
#define S5      F0 + 4
                 SHORT(1),
#define S4      S5 + 1
                 SHORT(1),
#define S3      S4 + 1
                 SHORT(1),
#define S2      S3 + 1
                 SHORT(1),
#define S1      S2 + 1
                 SHORT(1), 0,
#define S3A1    S1 + 2
                 SHORT(3), LONG(1), 0,
#define S5L1    S3A1 + 3
                 SHORT(5), LONG(1), 0,
#define S5H1    S5L1 + 3
                 SHORT(5), -SHORT(1), 0,
#define S5H1S1  S5H1 + 3
                 SHORT(5), -SHORT(1), SHORT(1), 0,
#define S7      S5H1S1 + 4
                 SHORT(7),  0,
#define S6C1    S7 + 2
                 SHORT(6), UCHAR(1), 0,
#define U8      S6C1 + 3
                 SHORT(8),  0,
#define S8C1    U8 + 2
                 SHORT(8), UCHAR(1), 0,
#define S3D4    S8C1 + 3
                 SHORT(3), DOUBLE(4),0, 
#define S1A1	S3D4 + 3
		SHORT(1), LONG(1), 0,
#define S1A1S1  S1A1 + 3
		SHORT(1), LONG(1), SHORT(1), 0
                };

#undef  UCHAR
#undef  SHORT
#undef  ADDRESS
#undef  LONG

    /* The following nonsensical way of getting the names is a circumvention
       of a bug in Lattice C 3.10.  Originally, I had a nice little structure
       with a pointer to the name and the editing code pointer.  All Lattice
       C would say about it was "Out of memory".  While this isn't quite as
       elegant, it has the virtue that it compiles. */
    static char names1[] =
                "INIT\0TERM\0CLEAR\0DOT\0VEC\0FILL\0MARK\0CMARK\0CHAR",
                names2[] =
                "ECHAR\0HLITE\0DHLITE\0MNUCURS\0TPROMPT\0QPLOT\0RPEN\0GOTEXT\0GOTEXTU",
                names3[] =
                "GOGRAPH\0WRSPLIT\0MODELINE\0COORDLINE\0SYNC\0REDRAW\0COMMAND\0XPCOMD\0BOXCLR",
                names4[] =
                "BOXSIZE\0BOXPUSH\0BOXPOP\0TEXT\0LINE\0DCURS\0CCURS\0BSHELL\0ASHELL",
                names5[] =
                "DIGTIZE\0OPENVP\0CLOSEVP\0VIEWPORT\0CONIN\0TABCFG\0DRAG\0LOPEN\0CLVP",
                names6[] =
                "REVEC\0STRING";
    static char pktdesc[] = {
                 /* INIT    */     I0,    0,   S2,   S3, S5H1S1,   F0,   S5,   S5,   S1,
                 /* ECHAR   */      0,   S2,   S2,   S2,      0,   S1,   S3,    0,    0,
                 /* GOGRAPH */     S1,   S1, S1A1S1, S1A1,   S3,   S3, S3A1,   S2,   S5,
                 /* BOXSIZE */   S5L1,   B0,   B0,   T0,   S5H1, S3A1,    0,   S2,   S2,
                 /* DIGTIZE */     S3,   S7,   S1,   S6C1,   S2,   S4,   S7, S3D4,   S4,
                 /* REVEC */       S3,   S1A1,
                };

    wrsplit((char)'<');
    if (sentinel)
        wrsplit((char)sentinel);
    wrsplit((char)'P');
    if (pkt->pfunc <= 0 || pkt->pfunc > sizeof(pktdesc)) {
        iptr = &codes[S5];
        send("KT func="); senddec(pkt->pfunc);
    } else {
        switch ((pkt->pfunc - 1) / 9) {
        case 0:       iptr = names1;  break;
        case 1:       iptr = names2;  break;
        case 2:       iptr = names3;  break;
        case 3:       iptr = names4;  break;
        case 4:       iptr = names5;  break;
        case 5:       iptr = names6;  break;
        }
        for (i = 0; i < (pkt->pfunc - 1) % 9; i++)
            while (*iptr++) {;}
        send(iptr);
        iptr = &codes[pktdesc[pkt->pfunc - 1]];
    }
    dptr = (char *)(&pkt->pfunc) + sizeof(pkt->pfunc);
    while ((count = (*iptr++ & 0xFF)) != 0) {
        radix = 10;
        if (count & 0x80) {
            count = -(count | ~0xFF);
            radix = 16;
        }
        len = (count & 3) + 1;
        count >>= 2;
        for (i = 0; i < count; i++, dptr += len) {
            send(" ");
            if (len == 3) {
                sendhex(4, ((short *)dptr)[1]);
                send(":");
                sendhex(4, ((short *)(dptr++))[0]);
            } else if (radix == 16) {
                if (len == 1)  {      /* double */
                    send(" ");
                    gcvt((*(double *)dptr),7,textstr); /* convert to string */
                    send(textstr);                
                    dptr += 7;        /* help bump the fetch pointer */
                } else {
                    for (j = len; j--; )
                        sendhex(2, dptr[j] & 0xFF);
                }
            } else {
                switch (len) {
                case 1:    senddec(*dptr & 0xFF);       break;
                case 2:    senddec(*(short *)dptr);     break;
                default:   sendlong(*(long *)dptr);     break;
                }
            }
        }
    }
    send("> ");
#undef  I0
#undef  B0
#undef  T0
#undef  F0
#undef  S5
#undef  S4
#undef  S3
#undef  S2
#undef  S1
#undef  S3A1
#undef  S5L1
#undef  S5H1
#undef  S6C1
#undef  U8
#undef  S8C1
#undef  S3D4
#undef  S1A1
}

#endif                                /* trace */



#ifdef flog

/* Send one character to our log file */
static void
/*FCN*/logchar(c)
  char c;
{
    char *p;

    if (logfile < (FILE *)0)
        return;
    if (c == '\r' || c == '\n') {
        fputc('\r',logfile);
        fputc('\n',logfile);
        return;
    } else {
        p = &c;
        fputc(*p,logfile);
    }
}


/* CLOSELOG -- If the log file is currently open, close it */
static void
/*FCN*/closelog()
{
    if (logfile >= (FILE *)0) {
        fclose(logfile);
        logfile = (FILE *)-1;
    }
}


/* send a hex number "v" with "n" digits just to the log file */
static
/*FCN*/sendlhex(n, v)
  short         n;
  unsigned short v;
{
    char    s[20];
    int     i;

    for (i =     n; i--; v >>= 4)
        s[i] = ((v & 15) >= 10 ? (v & 15) - 10 + 'A' : (v & 15) + '0');
    s[n] = '\0';
    sendlog(s);
}

/* send a decimal value just to the log file */
static
/*FCN*/sendldec(v)
  short v;
{
    if (v < 0) {
        logchar('-');
        v = -v;
    }
    if (v >=    10)
        sendldec(v / 10);
    logchar('0' + (int)(v % 10));
}


/* Send a string of text to the log file */
static void
/*FCN*/sendlog(s)
  char    *s;
{
    int c;

    if (logfile >= (FILE *)0)
        while (c = (*s++ & 0xFF))
            logchar(c);
}
#endif                                /* log */


/* ==================== added code below for PFILL =============*/

/* we scale logical polys to physical coords before running them
through lflood because this keeps us from drawing zillions of
unneeded vectors when we are zoomed in a lot - then we'll have
to clip the vectors which come from lflood to make them fit in
the target viewport, but that's another function. */
scalepoly(i)
	short i;
{
    short j, k;
#define VP vpinfo[i]
#define SP vpinfo[32]
	/* scale to pixel coords (using algorithm from iclip) */

    for (j = 0; j < poly.npoints; j++) {
    	poly.pt[j].x -= VP.lgxl;
    	poly.pt[j].x = poly.pt[j].x * (long)VP.wixdots / VP.vxdots;
		poly.pt[j].x += VP.vpxmin;
		poly.pt[j].y -= VP.lgyl;
        poly.pt[j].y = poly.pt[j].y * (long)VP.wiydots / VP.vydots;
		poly.pt[j].y += VP.vpymin;
    }

/* set up scaling factors for iclip to use for vectors - we don't
really want any scaling - just clipping */
	SP.lgxl = SP.vpxmin = VP.vpxmin;
	SP.lgyl = SP.vpymin = VP.vpymin;
	SP.lgxh = VP.vpxmin + VP.wixdots;	/* slave doesn't save vpxmax */
	SP.lgyh = VP.vpymin + VP.wiydots;	/* so we figure them out */
	SP.wixdots = SP.vxdots = VP.wixdots;
	SP.wiydots = SP.vydots = VP.wiydots;
/* point ecurwind (used by iclip) to the right entry in vpinfo[] */
        ecurwind = (long)&vpinfo[32].vpnumber;
#undef VP
#undef SP
}


/* a poly in the global struct poly is drawn in its (master)
viewport and any slaves of it, with proper scaling 
enter with index into vpinfo[] in i */
/*FCN*/redrawpoly(i)
  short i;
{
  short j, k;
  struct spolygon savep;
#define VP vpinfo[i]
#define SP vpinfo[j]
    hilight = 0;
    if (VP.slaves) {          /* is this a master? */
        /* if there are slaves, we have to save the original unscaled
        poly so we can rescale it for each slave */
        savep.npoints = poly.npoints;
        for (j = 0; j < poly.npoints; j++) {
            savep.pt[j].x = poly.pt[j].x;
            savep.pt[j].y = poly.pt[j].y;
        }
        for (j = 16; j < 31; j++) {		/* look for slaves */
	    if ((SP.master == VP.vpnumber) && SP.vpnumber) {
	        scalepoly(j);			/* scale for slave vp */
	        vectaker = pixlvec;
                drawpoly();
                /* restore saved poly */
                for (k = 0; k < poly.npoints; k++) {
                    poly.pt[k].x = savep.pt[k].x;
                    poly.pt[k].y = savep.pt[k].y;
                }
	    }
        }
    }
    scalepoly(i);				/* scale for master vp */
    vectaker = pixlvec;
    drawpoly();
#undef VP
#undef SP
}


static void
/*FCN*/drawpoly()
{
	if (hilight) {
		fillhigh = 0;
	}
	lflood();
}


/* draw a logical vector from lflood */
static void
/*FCN*/pixlvec()
{
    slvec(svec.f.x, svec.f.y, svec.t.x, svec.t.y, fprop);
}

/* draw a pixel vector in regular (non-XOR) ink */
static void pixvec()
{
	if (hilight) {
		if (fillhigh++ == 5) fillhigh = 0;
		if (fillhigh)
            		tadsvec(svec.f.x, ybias - svec.f.y, svec.t.x, 
				ybias - svec.t.y, fprop);
		else {
            		tadsvec(svec.f.x, ybias - svec.f.y, svec.t.x,
				ybias - svec.t.y, 
				property(fillcolor, NOGREY, TRUE));
		}
	}  else
            tadsvec(svec.f.x, ybias - svec.f.y, svec.t.x, ybias -
		svec.t.y, fprop);
}

/* draw a pixel vector in XOR ink */
static void
/*FCN*/pixxvec()
{
        tadsvec(svec.f.x, ybias - svec.f.y, svec.t.x, ybias - svec.t.y, 
		property(8, NOGREY, TRUE));
}


/* add a poly to the display list */
appendpoly(vpindex)
  short vpindex;
{
    short *aptr;
    short i, size;

#define VP vpinfo[vpindex]

    if (VP.overflow)       /* display list full! */
        return;

    size = 2 * ((poly.npoints * 2) + 2); /* bytes needed for this poly */

    if ((VP.ioffset + size) > DLSIZE) {
        VP.overflow = 1;
        return;
    }
    aptr = (short *)(VP.ioffset + VP.dlist); /* present end address */

    *(aptr++) = poly.pt[0].x | 0x8000; /* mark "moveonly" */
    *(aptr++) = poly.pt[0].y;
    *(aptr++) = (fprop + (poly.npoints << 12));
    *(aptr++) = 0xffff;               /* flag */
    for (i = 1; i < poly.npoints; i++) {
       *(aptr++) = poly.pt[i].x;
       *(aptr++) = poly.pt[i].y;
    }
    VP.ioffset += size;
}
#undef VP

#ifdef debug
void dumppoly()
{
    short i;
    send("<SPOLY n=");
    senddec(poly.npoints);
    for(i=0; i< poly.npoints; i++)
    {
        send(" ");
        senddec (poly.pt[i].x);
        send(",");
        senddec (poly.pt[i].y);
    }
    send("> ");
}

static void 
/*FCN*/dumpsvec()
{    
    send("<SVEC ");
    senddec(svec.f.x);
    send(",");
    senddec(svec.f.y);
    send(" ");
    senddec(svec.t.x);
    send(",");
    senddec(svec.t.y);
    send("> \n");
}
#endif

/*

	Rasta

	Software Polygon Fill

	Designed and implemented by Kelvin R. Throop in July of 1987,
	revised and updated, new and improved in December of 1988.
        Based on AutoCAD's polygon fill code, designed and implemented
	by John Walker in January of 1984.

		(C) Copyright 1987, 1988  Autodesk, Inc.

	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/


/* Take a polygon structure as input containing a clipped polygon in pixel
   coordinate space and output vectors to the subroutine address stored
   in VECTAKER. */

static void lflood()
{
	scrcoord minint, maxint, thisint;
	struct edg edge[MAXVERT];
	scrcoord inter[MAXVERT];
	int i, j, k, l, m, n, nvert, nedge, nint, bymax, bymin, horedge;
	scrcoord s, t, u, y;
        float frac;

#ifdef serial
#ifdef debug
dumppoly();
#endif
#endif

	nvert = poly.npoints;

	horedge = FALSE;	   /* No horizontal edges yet */
	assert(nvert <= MAXVERT);

	if (nvert == 1) {
	   svec.f.y = svec.t.y = poly.pt[0].y;
	   svec.f.x = svec.t.x = poly.pt[0].x;
	   (*vectaker)();
	   return;
	};

        /* find extents of polygon */
        poly.pymax = 0;
        poly.pymin = HIGHVAL;
        for (i = 0; i < nvert; i++) {
            poly.pymax = max(poly.pymax, poly.pt[i].y);
            poly.pymin = min(poly.pymin, poly.pt[i].y);
        }



	/*  Scan vertex table and calculate maximum and minimum Y extent
	    of each edge.  Build edge summary table.  */

	for (i = 0, j = 0; i < nvert; i++) {
	   k = (i + 1) % nvert;
	   if (vertex[i].y != vertex[k].y) {   /*  Ignore horizontal lines */
	      edge[j].ymax = max(vertex[i].y, vertex[k].y);
	      edge[j].ymin = min(vertex[i].y, vertex[k].y);
	      if (edge[j].ymax == vertex[i].y) {
		 l = i;
		 m = k;
	      } else {
		 l = k;
		 m = i;
	      }
	      edge[j].xpos = vertex[l].x;
	      edge[j].xend = vertex[m].x;
	      edge[j].slope = ((double) vertex[m].x - vertex[l].x) /
			      ((double) vertex[m].y - vertex[l].y);
	       j++;
	    } else {
	      horedge = TRUE;	   /* Remember horizontal edge */
	    }
	}
	nedge = j;

#ifdef DEBUG
#ifdef debug
        fprintf(logfile,"\nVertices:");
	for (i = 0; i < nvert; i++)
           fprintf(logfile,"\n  %d,%d  ", vertex[i].x, vertex[i].y);
        fprintf(logfile,"\nEdges:");
	for (i = 0; i < nedge; i++)
	   fprintf(logfile,
"\n  ymax=%d  ymin=%d  xpos=%d  slope=%9.4f  slantwid=%d\n",
	      edge[i].ymax, edge[i].ymin, edge[i].xpos, edge[i].slope);
        fprintf(logfile,"\nY range:  %d - %d", poly.pymin, poly.pymax);
#endif
#endif

	for (y = poly.pymax; y >= poly.pymin; y--) {


	   nint = 0;

	   /*  Scan edges and calculate X coordinates of intersections
	       with the scan line.  For the benefit of the plot-fill code,
	       which tries to connect consecutive scan lines with short
	       pen-down vectors, note when the outermost intersections
	       make transitions from one polygon edge to another.  */

	   minint = HIGHSCRC;
	   maxint = 0;
	   for (j = 0; j < nedge; j++)
	      if (y <= edge[j].ymax && y >= edge[j].ymin) {
		 if (y == edge[j].ymin)
		    thisint = inter[nint] = edge[j].xend;
		 else
		    thisint = inter[nint] =
		       edge[j].xpos - edge[j].slope * (edge[j].ymax - y);
		 if (thisint <= minint) {
		    minint = thisint;
		 }
		 if (thisint >= maxint) {
		    maxint = thisint;
		 }
		 nint++;
	       }

	   if (nint > 1) {	   /*  Skip if < 2 intersections  */

	      /*  Sort intersections  */

	      for (i = 0; i < (nint - 1); i++)
		 for (j = i + 1; j < nint; j++)
		    if (inter[i] >= inter[j]) {
		       s = inter[i];
		       inter[i] = inter[j];
		       inter[j] = s;
		    }

#ifdef DEBUG
#ifdef debug
              fprintf(logfile,"\nIntersections at y=%d: ", y);
	      for (i = 0; i < nint; i++)
                 fprintf(logfile,"  %d", inter[i]);
#endif
#endif

	      /*  Eliminate duplicates unless the duplicates result from a
		  vertex intersection which occurs at a local minimum or
		  maximum  (duplicates can only occur at vertices, i.e.,
		  the scan line is found to intersect both edges coming
		  into a vertex)  */

	      if (nint > 2)
		 for (i = 0; i < (nint - 1); i++)
		    if (inter[i] == inter[i + 1]) {
		       for (j = 0; j < nvert; j++) /*  Find vertex which did it  */
			  if (vertex[j].y == y && vertex[j].x == inter[i])
			     break;
		       s = vertex[(j == 0) ? nvert - 1 : j - 1].y;
		       t = vertex[(j + 1) % nvert].y;
		       u = vertex[j].y;
		       if (!((u > s && u > t) || (u < s && u < t))) {
			  for (j = i + 1; j < (nint - 1); j++)
			     inter[j] = inter[j + 1];
			  nint--;
#ifdef DEBUG
#ifdef debug
                          fprintf(logfile,"\nDuplicate removal:  ");
			  for (l = 0; l < nint; l++)
                             fprintf(logfile,"  %d", inter[l]);
#endif
#endif
		       }
		    }


	      /*  Just one more thing: if we still have more than
                  two intersections, there's the possibility that
		  two of them are actually the endpoints of a horizontal
		  edge which was discarded in the original edge scan.
		  If we discarded a horizontal edge, scan the vertex
		  table and see if two adjacent intersections are
		  coincident with that edge; if so, handle it
		  carefully.  */

	      if (horedge && (nint > 2)) {
		 for (i = 0; i < nvert; i++) {
		    if ((y == vertex[i].y) &&
			(y == vertex[(i + 1) % nvert].y)) {

                       /* Recompute this here because we'll get here
			  much less frequently than we make the prior
			  test.  Compiler only has to store the
                          subexpression "vertex[(i + 1) % nvert].y"
			  computed above if we need it.  */

		       l = vertex[i].x;
		       m = vertex[(i + 1) % nvert].x;

		       /* Now loop over the intersection table and test for
			  a pair of intersections coincident with a horizontal
			  edge.  Note the odd test we use to compensate for
                          direction-swapping induced by "whichway" plotter
			  motion compensation--this is cheaper than testing
			  whichway and taking a max and min.  When we detect
			  the inclusion of a horizontal edge in the
			  intersection table, the loop we use to excise it
			  guarantees that we always expunge the internal
			  vertex, preserving continuity of fill across
			  the horizontal edge of the polygon. */

		       for (k = 0; k < nint - 1; k++) {
			  if (((l == inter[k]) && (m == inter[k + 1])) ||
			      ((m == inter[k]) && (l == inter[k + 1]))) {

                             /* All right, we've found a horizontal
				edge.  How we handle this edge depends
				on whether it begins with a start fill
				(even) or end fill (odd) intersection.	*/

			     if ((k & 1) == 0) {

/*		  Case 1:  Start fill (even) intersection

        To understand this case better, let's consider a concrete
        example.  Suppose we're scanning down the polygon shown below,
        and that we're considering the line whose Y co-ordinate is
	coincident with the horizontal edge between vertices V2 and
	V3.  When we get here, the variables point to the geometry as
	indicated.


			      vertex[(i+1)%nvert]
					     |	 + V4
                          vertex[i]          |  / \
                             |               V /   \
                             V                /     \
   VERTEX TABLE  y----->     +---------------+	     *
                            /V2             V3       .\
                           /                           \
                          /  ^               ^       .  \
			 /   |		     |		 + V5
		     V1 +    |		     |	     .
			   l = v2[X]	   m = v3[X]
						     .

						     .
			  inter[k]	 inter[k+1]
			     |		     |	     .
			     |		     |
			     V		     V	     .
   INTERSECTION LIST	     +---------------+-------+
			     I1 	     I2      I3


	Remember that the relationship between the intersection list
	and the vertex table is tenuous and depends upon the shape of
        the polygon we're scan converting.  By this time the
	intersections in the table have been sorted by their X
	co-ordinates, duplicates have been expunged (unless they occur
	at local minima or maxima), and the order of the list depends
	on the current setting of whichway, which toggles to make pen
	plotters shuttle back and forth when filling rather than lift,
	move, and draw lines in the same direction.

	The fact that we know that the two intersections starting at
	inter[k] are coincident with the horizontal edge that starts
	at vertex[i] constrains the geometry, however.	What we now
	must decide is whether to discard intersection I2 from the
	intersection list.

        Pardon me.  No, no, I'm not the gardener, my name's Columbo.
        Lieutenant Columbo.  I'm from the police---LAPD, verticide.
        There's just one thing that's confusing me---could I borrow
	your pencil?---you know, my boss is really picky about
	details---we have all these reports we have to write, and if
        there's one little thing that's missing, he just doesn't let
	up until we get it down on paper---do you have a piece of
	paper I could borrow?---thanks.

        Well like I said, there's one thing that's confusing me.  What
	if you have a polygon like the one below?  Now your
	explanation above makes a lot of sense, but what if you have
	something like this?  Do you see what my problem is?  Now if
        you're scanning from left to right, you generate intersections
	I1 and I2 from vertices V1 and V2, and intersection I3 is
	generated by edge V3-V4.  But you see, and this is my problem,
	what if edge V1-V4 slopes up?  In that case, even though V3
        slopes down, we're still inside the polygon at V2, so you
	should have removed V2.

				   + V4
				  /|
				 / |
				/  |
			       /   |
			      /    |
			     /	   |
   VERTEX TABLE 	    /	   |
			   /	   |
			  /    V2  |
	y------>      V1 +-----+   |
                                \  |
                                 \ |
                                  \|
				   + V3

   INTERSECTION LIST	 +-----+---+
			I1    I2   I3

        So you see, and I've been thinking about this a lot---these
        things get on your mind and, you know, it's really hard to
	shake these things loose when they start to bother
        you---what's bothering me is that it really seems that what
	you have to do is look at the Y coordinates of the TWO
	vertices that adjoin the horizontal edge.  Just like the
	duplicate vertex case, the question is whether the horizontal
        edge occurs at a local maximum or minimum, or whether it's
        merely an inflection point---you know what that means, don't
	you---in the rise or fall from the two vertices that enclose
	it.  If it is, then we have to get rid of the second
        intersection, because it's inside the polygon.

	Now you see, suppose you had this polygon?

				     + V4
                               V2   / \
	y------>      V1 +-----+   /   *
                        /       \ /    .\
   VERTEX TABLE        /         + V3  . \
                      /                .  \
		  V6 +---------------------+ V5
				       .
   INTERSECTION LIST	 +-----+-+-----+
			I1    I2 I3    I4

	In this case, both V3 and V6 are in the same direction from V1
	and V2.  And you see what that means?  That means that V2 is a
	valid end point.  V2 has to remain in the intersection list.
        And that's the crucial detail.  That's what you have to
        remember.  And that's what you need to get the right answer.

	Thank you, Lieutenant.	So, to decide whether to ditch I2, we
	have to look at the edges of the polygon adjacent to the ends
	of the horizontal edge coincident with I2.  But although we
	know that I1 and I2 were caused by V2 and V3, the whichway
	alternation keeps us from knowing which caused which.  But
	since we only care about whether the horizontal edge forms a
        local minimum or maximum, we don't really care about the order
	relationship between the intersection list and the vertex
	table--we just need to be able to find the two vertices
	adjacent to the horizontal edge.  Fortunately, the lieutenant
	scribbled down equations to find those vertices.

	Based on the first picture, we can locate the precise vertices
	by comparing l, the X co-ordinate of V2, with the X value
	saved in the intersection table entry inter[k].  If l equals
	inter[k], the picture is as drawn; otherwise, the intersection
	table is in reverse order: l equals inter[k+1], I1 corresponds
	to V3, I2 to V2, and we want to examine V1 to decide whether
	to keep I2.

	Consequently, the index of the vertex table entries whose Y
	co-ordinate we should test are given by:

	   vertex[(i+2)%nvert], 	 if l == inter[k]     (V4)
	   vertex[((i+nvert)-1)%nvert],  if l == inter[k+1]   (V1)

	Note how the way the expression used to address V1 is written
	handles the case where V2 is the first vertex in the table and
	the V2-V1 segment is the wrap-around closing segment of the
	polygon.

	So, we should delete I2 from the vertex table unless the
	Y component of V4 in the first picture minus the Y component
	of V2 and V3 (current y) has the same sign as the Y component
	of V2 and V3 (current y) minus the Y component of V1.  */

				s = vertex[((i + 2) % nvert)].y;
				t = vertex[((i + nvert) - 1) % nvert].y;
				if (((s <= y) && (t >= y)) ||
				    ((s >= y) && (t <= y))) {

				   for (n = k + 1; n < (nint - 1); n++)
				      inter[n] = inter[n + 1];
				   nint--;
#ifdef DEBUG
#ifdef debug
				   fprintf(logfile,
                                     "\nHorizontal edge case 1%c: ",
                                     s < y ? 'a' : 'b');
				   for (l = 0; l < nint; l++)
                                      fprintf(logfile,"  %d", inter[l]);
#endif
#endif
				}
			     } else { /* ((k & 1) == 1) */

			     /* Case 2: The index of the first vertex
				of the edge is odd, and hence we will
				normally stop drawing at that point.
				Now if we were legitimately drawing
				at the start of a horizontal edge, we
				know unconditionally that we should continue
				to draw across that edge--it is, after all,
				part of the polygon.  In this case, we
				excise the first vertex of the edge and
				shift all of the edges to the left. */

				for (n = k; n < (nint - 1); n++)
				   inter[n] = inter[n + 1];
				nint--;
#ifdef DEBUG
#ifdef debug
				fprintf(logfile,
                                   "\nHorizontal edge case 2: ");
				for (l = 0; l < nint; l++)
                                   fprintf(logfile,"  %d", inter[l]);
#endif
#endif
			     }
			  }
		       }
		    }
		 }
	      }

	      /*  Draw lines between pairs of intersections  */

	      for (i = 0; i < (nint - 1); i += 2) {
		 svec.f.y = svec.t.y = y;
		 svec.f.x = inter[i];
		 svec.t.x = inter[i + 1];
                 if (svec.f.x < 0 && svec.t.x < 0)
                     continue;
                 if (svec.f.x < 0) svec.f.x = 0;
                 if (svec.t.x < 0) svec.t.x = 0;
		 (*vectaker)();
	      }
	   }
	}
        /* now clean up the image by drawing the actual edges of the
        polygon! This fixes thin polylines. */
        for (i = 0; i < nvert; i++) {
            j = (i + 1) %nvert;
            svec.f.x = vertex[i].x;
            svec.f.y = vertex[i].y;
            svec.t.x = vertex[j].x;
            svec.t.y = vertex[j].y;
#ifdef serial
#ifdef debug
dumpsvec();
#endif
#endif
/* if a vector overlaps into negative coords, and it isn't horizontal,
we have to do a good job of clipping it to the left edge of the coordinate
system */
            if (svec.f.x < 0 && svec.t.x > 0) {
                frac = (float)svec.t.x / (float)(svec.t.x - svec.f.x);
#ifdef sterm
fprintf(sterm," frac: %9.4f ",frac);
#endif
                svec.f.x = 0;
                svec.f.y = svec.t.y + frac * (svec.f.y - svec.t.y);
#ifdef sterm
fprintf(sterm,"new f.y:%x, t.y:%x\n", svec.f.y, svec.t.y);
#endif
            }
            if(svec.f.x >= 0 && svec.t.x >= 0) {
#ifdef serial
#ifdef debug
                dumpsvec();
#endif
#endif
                (*vectaker)();
            }
        }
}

