/* 
       
       PLADI.C

    Sample Unix AutoCAD Plotter ADI driver.


*/
/*	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/

#include <stdio.h>
#include "uadi.h"


main(argc, argv)
  int argc;
  char *argv[];
{

   int fdr, fdw, nread, nwrite;
   short heart_beat = 1;
   struct plipkt plipkt;	      /* initialization packet */
   struct plpkt plpkt;		      /* data packet */
   int temp;

#ifndef VAX
   fdr = atoi(argv[1]); 	      /* get read file descriptor number */
   fdw = atoi(argv[2]); 	      /* get write file descriptor number */
#else
   sscanf(argv[1], "%d%c", &temp);
   ppmap_segment(temp, "adipp");
   fdr = ADIPLR;
   fdw = ADIPLW;
#endif

   write(fdw, &heart_beat, 2);	      /* send heart-beat */

   nread = read(fdr, &plipkt, PLIPKTSIZE); /* read initialization packet */
   if (nread != PLIPKTSIZE) {
      fprintf(stderr, "PLADI: read wrong number of bytes %d\n", nread);
      exit(1);
   }
   fprintf(stderr, "Version level = %d, xmax,ymax = %.5g,%.5g \
xrel,yrel=%.5g,%.5g\n\
xinc,yinc=%.5g,%.5g linewidth=%.5g npens=%d maxspeed=%d linetypes=%d\n",
      plipkt.level, plipkt.plxmax, plipkt.plymax, plipkt.plx, plipkt.ply,
      plipkt.plxinc, plipkt.plyinc, plipkt.pllwid, plipkt.plnpens,
      plipkt.plspdmax, plipkt.plltmax);

   plipkt.code = 1;		      /* return good status */
   nwrite = write(fdw, &plipkt, PLIPKTSIZE);
   if (nwrite != PLIPKTSIZE) {
      fprintf(stderr, "PLADI: write wrong number of bytes %d\n", nread);
      exit(1);
   }
   

   for ( ;; ) {
      switch (nread = read(fdr, &plpkt, PLPKTSIZE)) {
      case -1:
         fprintf(stderr, "PLADI: read error\n");
	 exit(1);
      case 0:
         fprintf(stderr, "PLADI: unexpected end of file.\n");
	 exit(1);
      default:
         fprintf(stderr, "PLADI: read wrong number of bytes: %d\n", nread);
	 exit(1);
      case PLPKTSIZE:		      /* correct number of bytes */
	 switch (plpkt.code) {
	 case PLOTSIZE:
            fprintf(stderr, "Plotter steps X,Y = %u,%u\n", plpkt.x, plpkt.y);
	    break;
	 case MOVE:
            fprintf(stderr, "Move to %u,%u\n", plpkt.x, plpkt.y);
	    break;
	 case DRAW:
            fprintf(stderr, "Draw to %u,%u\n", plpkt.x, plpkt.y);
	    break;
	 case NEWPEN:
            fprintf(stderr, "New pen number %d\n", plpkt.x);
	    break;
	 case NEWSPEED:
            fprintf(stderr, "Select speed %d\n", plpkt.x);
	    break;
	 case NEWLTYPE:
            fprintf(stderr, "Set line type %d\n", plpkt.x);
	    break;
	 case PENRAISE:
            fprintf(stderr, "Pen raise.\n");
	    break;
	 case ABORTPLOT:
            fprintf(stderr, "Abort plot.\n");
	    break;
	 case ENDPLOT:
            fprintf(stderr, "End plot.\n");
	    exit(0);
	 default:
            fprintf(stderr, "Bad code = %d, x,y = %u,%u\n", plpkt.code, plpkt.x, plpkt.y);
	    break;
	 }
	 break;
      }
   }
}
