/* 
       
       PPADI.C

    Sample Unix AutoCAD Printer Plotter ADI driver.

      The read file number is passed as command line argument 1, and
      the write file number is passed as command line argument 2.

    Compile and link this program with:
      
      cc -o ppadi ppadi.c

*/
/*	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/

#include <stdio.h>
#include "uadi.h"

main(argc, argv)
  int argc;
  char *argv[];
{
   int fdr, fdw, nread, nwrite;
   short heart_beat = 1;
   struct ppipkt ppipkt;	      /* initialization packet */
   struct pppkt pppkt;		      /* data descriptor packet */
   char data[2000];
   int temp;
   int rr;

#ifndef VAX
   fdr = atoi(argv[1]); 	      /* get read file descriptor number */
   fdw = atoi(argv[2]); 	      /* get write file descriptor number */

#else
   sscanf(argv[1], "%d%c", &temp);
   ppmap_segment(temp, "adipp");
   fdr = ADIPPR;
   fdw = ADIPPW;
#endif
   write(fdw, &heart_beat, 2);	      /* send heart-beat */

   /* Read begin plot packet */
   nread = read(fdr, &ppipkt, PPIPKTSIZE);
   if (nread != PPIPKTSIZE || ppipkt.code != 1) {
      printf("PPADI: bad initialization packet read. Size = %d\n", nread);
      exit(1);
   }
   printf("Begin Printer Plot: Level = %d, xdots = %u, ydots = %u\n", 
      ppipkt.level, ppipkt.xdots, ppipkt.ydots);

   /* Select 1 bit or 4 bits per pixel. */
   ppipkt.color = 0;		      /* 0=Monochrome, 1=Color */   
   if ((nwrite = write(fdw, &ppipkt, PPIPKTSIZE)) != PPIPKTSIZE) {
      printf("PPADI: Bad write. %d\n", nwrite);
      exit(1);
   }

   for ( ;; ) {

      switch (nread = read(fdr, &pppkt, PPPKTSIZE)) {
      case -1:
         printf("PPADI: read error\n");
	 exit(1);
      case 0:
         printf("PPADI: unexpected end of file.\n");
	 exit(1);
      default:
         printf("PPADI: read wrong number of bytes: %d\n", nread);
	 exit(1);
      case PPPKTSIZE:		      /* correct number of bytes */
	 switch (pppkt.code) {
	 case PPPLOTDATA:
            printf("Printer plot data: count = %d y = %u\n", pppkt.count, pppkt.y);
	    if (pppkt.count > sizeof data) {
               printf("Kaboom! data size too large.\n");
	       exit(1);
	    }
	    rr = pppkt.count;
	    if ((nread = read(fdr, data, rr)) != rr) {
               printf("PPADI: read wrong number of bytes: %d\n", nread);
	       exit(1);
	    } else
               printf("Read %u data bytes.\n", nread);
	    break;
	 case PPENDPLOT:
            printf("End printer plot.\n");
	    exit(0);
	 case PPABORTPLOT:
            printf("Abort printer plot.\n");
	    break;
	 default:
            printf("Bad code = %d, count = %u y = %u\n",
	       pppkt.code, pppkt.count, pppkt.y);
	    break;
	 }
	 break;
      }
   }
}
