/* Next available MSG number is   1 */
/*
      (C) Copyright 1987, 1988, 1989 by Autodesk, Inc.

	This program is copyrighted by Autodesk, Inc. and is
	furnished pursuant to a software license agreement.  The
	source code of this program may not be distributed or
	published in any form by the licensee.	Licensee is free to
	incorporate this code in object form in derivative works
	providing such derivative works are solely intended to work
	with Autodesk, Inc. products.  Said derivative works,
	incorporating object versions of this program may be
	distributed without royalty at the discretion of the licensee.

	Autodesk makes no warranties, express or implied, as to the
	correctness of this code or any derivative works which
        incorporate it.  Autodesk provides this code on an ``as-is''
	basis and explicitly disclaims any liability, express or
	implied, for errors, omissions, and other problems in the
	code, including consequential and incidental damages.

*/

/*     UADI.H

    Unix ADI driver packet definitions.

*/

#include "rdadi.h"  /* Rendering ADI packet definitions */

#ifndef real
#define real double
#endif

/* Unix ADI printer plotter initialization packet. */
struct ppipkt {
   short code;
   short level;
   unsigned short xdots;
   unsigned short ydots;
   short color;
};
#define PPIPKTSIZE (sizeof(struct ppipkt))

/* Unix ADI printer plotter data descriptor packet */
struct pppkt {
   short code;			    /* command code */
   short count; 		    /* byte count of following data */
   unsigned short y;		    /* y value */
};
#define PPPKTSIZE (sizeof(struct pppkt))

/* Values contained in ppipkt.code and pppkt.code */

#define PPBEGINPLOT 1
#define PPPLOTDATA  2
#define PPENDPLOT   3
#define PPABORTPLOT 4



/* Unix ADI plotter initialization packet */
struct plipkt {
   short code;			    /* command code */
   short level; 		    /* ADI version level */
   real plxmax, plymax; 	    /* Max plot dimensions */
   real plx, ply;		    /* Plot dim. relative to origin */
   real plxinc, plyinc; 	    /* x and y plotter increments/inch */
   real pllwid; 		    /* Line width */
   short plnpens;		    /* number of pens */
   short plspdmax;		    /* Maximum pen speed */
   short plltmax;		    /* number of hardware line types */
};
#define PLIPKTSIZE (sizeof(struct plipkt))


/* Unix ADI plotter data packet. */
struct plpkt {
   short code;			    /* command code */
   unsigned short x, y; 	    /* Arg or x and y values */
};
#define PLPKTSIZE (sizeof(struct plpkt))

/* Values contained in plipkt.code and plpkt.code */

#ifndef BEGINPLOT
#define BEGINPLOT   1		   /* Begin plot */
#define ENDPLOT     2		   /* End of plot */
#define MOVE	    3		   /* Move with pen up */
#define DRAW	    4		   /* Draw with pen down */
#define NEWPEN	    5		   /* Select new pen */
#define NEWSPEED    6		   /* Select new speed */
#define NEWLTYPE    7		   /* New line font */
#define PENRAISE    8		   /* Manual pen change */
#define ABORTPLOT   9		   /* Plot terminated by user */
#define PLOTSIZE   10		   /* Return total plotter steps */
#endif
 

/* Unix digitizer/pointer initialization ADI driver packet */
struct dgipkt {
   short dgcode;		      /* initialization status */
   short level; 		      /* ADI version level */
   short tablet;		      /* driver wants tablet mode */
   short pauses;		      /* tablet returns no sample */   
};
#define DGIPKTSIZE (sizeof(struct dgipkt))


/* Unix digitizer/pointer ADI driver packet */
struct dgpkt {
   short dgcode;
   short buttn;
   short x;
   short y;
};
#define DGPKTSIZE (sizeof(struct dgpkt))

/* Codes passed to driver in dgipkt.dgcode and dgpkt.code */
#define DGINIT	1		      /* initialize request */
#define DGSENSE 2		      /* return digitizer input */
#define DGTERM	3		      /* terminate digitizer driver */

/* Values returned by digitizer ADI driver in dgpkt.dgcode */
#ifndef P_NONE
#define P_NONE	0		      /* No data sample */
#define P_COORD 2		      /* sample is coordinate pair */
#define P_POINT 3		      /* selected point returned */
#define P_BUTTN 4		      /* button number returned */
#define P_BUTTP 5		      /* button selected point returned */
#endif


/* Unix ADI Renderer definitions.

   These definitions represent the UNIX variations on rendering packet
   request definitions as they appear in RDADI.H.
   These are needed because UNIX ADI uses pipes instead of common
   memory to transfer packet requests.  If your UNIX ADI rendering
   driver uses the functions in rdupkt.c to manage packet transfer,
   then it will not be necessary to use these definitions. */

struct rdu_init {		   /* Initialisation request (all reply cells) */
				   /* Also used for resize request */
	unsigned short flags;	   /* Initialization flags */
	short xdots, ydots;	   /* Total screen size in X and Y */
	unsigned short pixwid, pixhgt;/* Pixel width and pixel height */
	short ncolour;		   /* Number of rgb colour values,
				      0x8000 bit = stereo display */
	unsigned short maxintens;  /* Maximum intensity */
	unsigned short nshades;    /* Number of shades per colour */
	short name_len;		   /* length of Driver name string */
	short rdcfg_len; 	   /* length of Device configuration block */
};

struct rdu_fname {		   /* Filename */
	unsigned short flags;	   /* flags - see below */
	short fname_len;	   /* length of name entered by user */
	short errmsg_len; 	   /* length of error message */
	short ftitle_len;	   /* length of title for file request */
};

#define RD_BATCH_SIZE 2040         /* Good size for the SUN systems */

struct rdu_batch {
	short nbr_packets;
        char  buf[RD_BATCH_SIZE];
};

/*  Master Unix rendering display driver request packet union  */

union rdu_mstr {
	struct rdu_init	 rinit;	   /* Initialise */
	struct rd_start  rstart;   /* Start rendering */
	struct rd_poly	 rpoly;	   /* polygon vertices */
	struct rd_cmap	 rcmap;	   /* colour map entry */
	struct rdu_fname rfname;   /* filename */
	struct rd_crange rcrange;  /* shading range */
	struct rd_cpoly  rcpoly;   /* continuous tone polygon */
	struct rdu_batch rbatch;   /* batch of polygons */
};

/* Unix ADI Rendering driver request/reply packet definition. */
struct rdupkt {
   short rdfunc;                      /* Function / status */
   short byte_cnt;                    /* Number of bytes in record to follow. */
   union rdu_mstr rpkt;
};

/* Unix ADI Renderer request packets are passed in two parts,
   the first being the function code and the length of the packet data,
   the second beinbg the packet data. */
#define RDPKTHDRSIZE (2*sizeof(short))
#define RDPKTDATSIZE (sizeof(union rd_mstr))

/* Unix ADI Renderer status return code size */
#define RDPKTRETSIZE (sizeof(rdfunc))

#ifdef VAX
#define read pread
#define write pwrite
#define ADIPPR 70
#define ADIPPW 71
#define ADIPLR 72
#define ADIPLW 73
#define ADIDGR 74
#define ADIDGW 75
#define ADIRDR 76
#define ADIRDW 77
#endif
